import logging

from django.contrib import admin

from ..models import (
    INSPIRE_EU_THEMES,
    ApplicationSchema,
    CodeList,
    CodeListValue,
    Namespace,
    Status,
    Theme,
    UnitOfMeasure,
)

log = logging.getLogger(__name__)


@admin.register(ApplicationSchema)
class ApplicationSchemaAdmin(admin.ModelAdmin):
    pass


@admin.register(CodeList)
class CodeListAdmin(admin.ModelAdmin):
    pass


@admin.register(CodeListValue)
class CodeListValueAdmin(admin.ModelAdmin):
    pass


@admin.register(Namespace)
class NamespaceAdmin(admin.ModelAdmin):
    pass


@admin.register(Status)
class StatusAdmin(admin.ModelAdmin):
    pass


@admin.register(Theme)
class ThemeAdmin(admin.ModelAdmin):
    pass


@admin.register(UnitOfMeasure)
class UnitOfMeasureAdmin(admin.ModelAdmin):
    pass


if "cadastral_parcels" in INSPIRE_EU_THEMES and INSPIRE_EU_THEMES["cadastral_parcels"]:
    from ..models.cadastral_parcels.admin import *  # noqa

if "buildings" in INSPIRE_EU_THEMES and INSPIRE_EU_THEMES["buildings"]:
    from ..models.buildings.admin import *  # noqa



"""*********
Buildings
*********

Definition
==========
Geographical location of buildings specifications

Description
===========
A building is a covered facility usable for the protection of humans animals things or the production of economic
goods A building refers to any structure permanently constructed or erected on its site Information on location of
buildings may be supplied as points or with the actual basic form of the building Usually buildings are part of
cadastre On the local level buildings are available within the large scale cadastral maps or cadastral data sets
and are geometrically represented as surfaces

Most buildings can be identified geocoded by address separate theme in INSPIRE

References
==========
    * `Data Specifications <https://inspire.ec.europa.eu/Themes/126/2892>`_
    * `INSPIRE Data Specification on Buildings – Technical Guidelines <https://inspire.ec.europa.eu/id/document/tg/bu>`_
    * `Theme description from Registry <https://inspire.ec.europa.eu/theme/bu>`_
    * `Building Base UML <https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:3:2:1:7883>`_

    .. figure:: ../../../docs/_static/img/buildings_base.png
        :align: center
        :target: https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:3:2:1:7883

    * `Building2D Base UML <https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:3:2:2:7911>`_

    .. figure:: ../../../docs/_static/img/buildings2D.png
        :align: center
        :target: https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:3:2:2:7911

    * `Building3D Base UML <https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:3:2:3:7916>`_

    .. figure:: ../../../docs/_static/img/buildings3D.png
        :align: center
        :target: https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:3:2:3:7916


Models
======

.. automodule:: inspire_eu.models.buildings.abstract
   :members:
   :inherited-members:
   :exclude-members: __init__,clean,clean_fields,full_clean,get_deferred_fields,refresh_from_db,save,save_base,serializable_value,validate_unique

"""  # noqa

import logging

from django.contrib.gis.db import models
from django.utils.translation import ugettext_lazy as _

from .. import INSPIRE_EU_THEMES

log = logging.getLogger(__name__)


if "buildings" in INSPIRE_EU_THEMES and INSPIRE_EU_THEMES["buildings"]:
    from ...models import INSPIRE_EU_DEFAULT_SRID, BaseInspireEUModel
    from ...models.abstract import AbstractGeographicalName
    from ..cadastral_parcels import CadastralParcel
    from .abstract import (
        AbstractBuildingCurrentUse,
        AbstractBuildingExtended2D,
        AbstractBuildingGeometry2D,
        AbstractBuildingNature,
        AbstractConstruction,
        AbstractDocument,
        AbstractElevation,
        AbstractExternalReference,
        AbstractHeightAboveGround,
        AbstractOtherConstruction,
    )

    class Building(
        BaseInspireEUModel,
        AbstractConstruction,
        AbstractBuildingExtended2D,
        AbstractBuildingGeometry2D,
    ):
        """Building

        Definition
        ----------
        A Building is an enclosed construction above and/or underground, used or intended for the shelter of humans,
        animals or things or for the production of economic goods. A building refers to any structure permanently
        constructed or erected on its site.
        """

        parent = models.ForeignKey(
            "self",
            blank=True,
            null=True,
            on_delete=models.PROTECT,
            related_name="parts",
        )
        cadastral_parcels = models.ManyToManyField(CadastralParcel)
        is_building_part = models.BooleanField(
            help_text=_("Is it a BuildingPart?"),
            default=False,
        )

        class Meta:
            verbose_name = _("Building")
            verbose_name_plural = _("Buildings")

        def __str__(self):
            return self.local_id

    class BuildingDocument(BaseInspireEUModel, AbstractDocument):
        """Building Document

        Definition
        ----------
        This data types provides the address where the document may be found and a minimum set of metadata elements
        considered as necessary to exploit the document.

        """

        building = models.ForeignKey(Building, on_delete=models.PROTECT)

        class Meta:
            verbose_name = _("Building Document")
            verbose_name_plural = _("Building Documents")
            ordering = ["building", "document_link"]

        def __str__(self):
            return "%s %s" % (self.building, self.document_link)

    class BuildingElevation(BaseInspireEUModel, AbstractElevation):
        """Building Elevation

        Definition
        ----------
        Vertically-constrained dimensional property consisting of an absolute measure referenced to a well-defined
        surface which is commonly taken as origin (geoïd, water level, etc.).

        Description
        ------------
        Source: adapted from the definition given in the data specification of the theme Elevation.
        """

        building = models.ForeignKey(Building, on_delete=models.PROTECT)

        # DirectPosition : Public Class
        # https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=1:3:6:1:1:985
        coordinate = models.PointField(
            srid=INSPIRE_EU_DEFAULT_SRID,
            blank=True,
            null=True,
        )
        dimension = models.IntegerField(blank=True, null=True)

        class Meta:
            verbose_name = _("Building Elevation")
            verbose_name_plural = _("Building Elevations")
            ordering = ["building", "dimension"]

        def __str__(self):
            return "%s %s" % (self.building, self.dimension)

    class BuildingExternalReference(BaseInspireEUModel, AbstractExternalReference):
        """Building External Reference

        Description
        -----------
        Reference to an external information system containing any piece of information related to the spatial object.

        EXAMPLE 1: Reference to another spatial data set containing another view on buildings; the externalReference may
                be used for instance to ensure consistency between 2D and 3D representations of the same buildings
        EXAMPLE 2: Reference to cadastral or dwelling register. The reference to this register may enable to find legal
                information related to the building, such as the owner(s) or valuation criteria (e.g. type of heating,
                toilet, kitchen)
        EXAMPLE 3: Reference to the system recording the building permits. The reference to the building permits may be
                used to find detailed information about the building physical and temporal aspects.

        """  # noqa

        building = models.ForeignKey(Building, on_delete=models.PROTECT)

        class Meta:
            verbose_name = _("Building External Reference")
            verbose_name_plural = _("Building External References")
            ordering = ["building", "reference"]

        def __str__(self):
            return "%s %s" % (self.building, self.reference)

    class BuildingHeightAboveGround(BaseInspireEUModel, AbstractHeightAboveGround):
        """Height above ground

        Definition
        ----------
        Vertical distance (measured or estimated) between a low reference and a high reference.

        References
        ----------
        https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:3:2:1:7898

        """

        building = models.ForeignKey(Building, on_delete=models.PROTECT)

        class Meta:
            verbose_name = _("Building Height Above Ground")
            verbose_name_plural = _("Building Heights Above Ground")
            ordering = ["building", "height_reference"]

        def __str__(self):
            return "%s %s %s" % (self.building, self.height_reference, self.value)

    class BuildingNature(BaseInspireEUModel, AbstractBuildingNature):
        building = models.ForeignKey(Building, on_delete=models.PROTECT)

        class Meta:
            verbose_name = _("Building Nature")
            verbose_name_plural = _("Building Natures")
            ordering = ["building", "nature"]

        def __str__(self):
            return "%s %s" % (self.building, self.nature)

    class BuildingCurrentUse(BaseInspireEUModel, AbstractBuildingCurrentUse):
        building = models.ForeignKey(Building, on_delete=models.PROTECT)

        class Meta:
            verbose_name = _("Building Current Use")
            verbose_name_plural = _("Building Current Uses")
            ordering = ["building", "current_use"]

        def __str__(self):
            return "%s %s %s" % (self.building, self.current_use, self.percentage)

    class BuildingGeographicalName(BaseInspireEUModel, AbstractGeographicalName):
        building = models.ForeignKey(Building, on_delete=models.PROTECT)

        class Meta:
            verbose_name = _("Geographical Name")
            verbose_name_plural = _("Geographical Names")
            ordering = ["building", "source_of_name"]

        def __str__(self):
            return "%s %s" % (self.building, self.source_of_name)

    class OtherConstruction(BaseInspireEUModel, AbstractOtherConstruction):
        building = models.ForeignKey(
            Building,
            on_delete=models.PROTECT,
            blank=True,
            null=True,
        )
        cadastral_parcels = models.ManyToManyField(CadastralParcel)
        geometry = models.MultiPolygonField(
            srid=INSPIRE_EU_DEFAULT_SRID,
            blank=True,
            null=True,
            help_text=_("2D or 2.5D geometric representation"),
        )

        class Meta:
            verbose_name = _("Other Construction")
            verbose_name_plural = _("Other Constructions")



# Generated by Django 2.2.19 on 2021-03-17 17:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('inspire_eu', '0002_inspire_eu_core'),
    ]

    operations = [
        migrations.RenameField(
            model_name='cadastralparcel',
            old_name='national_cadastal_reference',
            new_name='national_cadastral_reference',
        ),
    ]



import logging

from django.contrib.gis import admin

from . import CadastralParcel, CadastralZoning

log = logging.getLogger(__name__)


@admin.register(CadastralZoning)
class CadastralZoningAdmin(admin.OSMGeoAdmin):
    pass


@admin.register(CadastralParcel)
class CadastralParcelAdmin(admin.OSMGeoAdmin):
    pass



# Generated by Django 2.2.19 on 2021-02-25 12:41

import django.contrib.gis.db.models.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('inspire_eu', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ApplicationSchema',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(db_index=True, max_length=32)),
                ('slug', models.CharField(blank=True, db_index=True, max_length=32)),
                ('link', models.URLField()),
                ('version', models.SmallIntegerField(blank=True, default=0)),
                ('label', models.CharField(max_length=128)),
                ('definition', models.TextField(blank=True)),
                ('description', models.TextField(blank=True)),
            ],
            options={
                'verbose_name': 'Application Schema',
                'verbose_name_plural': 'Application Schemes',
                'ordering': ['code', 'label'],
            },
        ),
        migrations.CreateModel(
            name='Building',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('local_id', models.CharField(help_text='A local identifier, assigned by the data provider. The local identifier is unique within the namespace, that is no other spatial object carries the same unique identifier.<br />NOTE It is the responsibility of the data provider to guarantee uniqueness of the local identifier within the namespace.', max_length=32)),
                ('version_id', models.CharField(blank=True, help_text='The identifier of the particular version of the spatial object, with a maximum length of 25 characters. If the specification of a spatial object type with an external object identifier includes life-cycle information, the version identifier is used to distinguish between the different versions of a spatial object. Within the set of all versions of a spatial object, the version identifier is unique.<br />NOTE The maximum length has been selected to allow for time stamps based on ISO 8601, for example, "2007-02-12T12:12:12+05:30" as the version identifier.<br />NOTE 2 The property is void, if the spatial data set does not distinguish between different versions of the spatial object. It is missing, if the spatial object type does not support any life-cycle information.', max_length=25)),
                ('begin_lifespan_version', models.DateTimeField(help_text='Registered area value giving quantification of the area projected on the horizontal plane of the cadastral parcel.')),
                ('end_lifespan_version', models.DateTimeField(blank=True, help_text='Date and time at which this version of the spatial object was superseded or retired in the spatial data set.', null=True)),
                ('date_of_construction_beginning', models.DateTimeField(blank=True, help_text='Beginning date of construction', null=True)),
                ('date_of_construction_end', models.DateTimeField(blank=True, help_text='End of date of construction', null=True)),
                ('date_of_construction_any_point', models.DateTimeField(blank=True, help_text='Any point of date of construction', null=True)),
                ('date_of_demolition_beginning', models.DateTimeField(blank=True, help_text='Beginning date of demolition', null=True)),
                ('date_of_demolition_end', models.DateTimeField(blank=True, help_text='End of date of demolition', null=True)),
                ('date_of_demolition_any_point', models.DateTimeField(blank=True, help_text='Any point of date of demolition', null=True)),
                ('date_of_renovation_beginning', models.DateTimeField(blank=True, help_text='Beginning date of last major renovation', null=True)),
                ('date_of_renovation_end', models.DateTimeField(blank=True, help_text='End of date of last major renovation', null=True)),
                ('date_of_renovation_any_point', models.DateTimeField(blank=True, help_text='Any point of date of last major renovation', null=True)),
                ('number_of_dwellings', models.SmallIntegerField(blank=True, help_text='A dwelling is a residential unit which may consist of one or several rooms designed for the occupation of households.<br />NOTE: In the data sets including building units, a dwelling is a residential building unit or, only when that building has no building units, a residential building<br />EXAMPLES: a single building dwelling could be a detached or semi-detached house. A block of flats will contain multiple dwellings determined by the number of individual flats.', null=True)),
                ('number_of_building_units', models.SmallIntegerField(blank=True, help_text='Number of building units in the building. A BuildingUnit is a subdivision of Building with its own lockable access from the outside or from a common area (i.e. not from another BuildingUnit), which is atomic, functionally independent, and may be separately sold, rented out, inherited, etc.<br />Building units are spatial objects aimed at subdividing buildings and/or building parts into smaller parts that are treated as separate entities in daily life. A building unit is homogeneous, regarding management aspects.<br />EXAMPLES: It may be e.g. an apartment in a condominium, a terraced house, or a shop inside a shopping arcade.<br />NOTE 1: According to national regulations, a building unit may be a flat, a cellar, a garage or set of a flat, a cellar and a garage.<br />NOTE 2: According to national regulation, a building that is one entity for daily life (typically, a single family house) may be considered as a Building composed of one BuildingUnit or as a Building composed of zero BuildingUnit.', null=True)),
                ('number_of_floors_above_ground', models.SmallIntegerField(blank=True, help_text='Number of floors above ground.', null=True)),
                ('geometry', django.contrib.gis.db.models.fields.MultiPolygonField(blank=True, help_text='2D or 2.5D geometric representation', null=True, srid=4326)),
                ('reference_geometry', models.BooleanField(help_text='The geometry to be taken into account by view services, for portrayal.<br />NOTE 1: In case of multiple representation by point and by surface, it is generally recommended to provide the surface as reference geometry.<br />NOTE 2: The geometric representation whose referenceGeometry is true may also be used preferably for spatial queries by download services (WFS) or by Geographical Information System (GIS).')),
                ('horizontal_geometry_estimated_accuracy', models.FloatField(blank=True, help_text='The estimated absolute positional accuracy of the (X,Y) coordinates of the building geometry, in the INSPIRE official Coordinate Reference System. Absolute positional accuracy is defined as the mean value of the positional uncertainties for a set of positions where the positional uncertainties are defined as the distance between a measured position and what is considered as the corresponding true position.<br />NOTE: This mean value may come from quality measures on a homogeneous population of buildings or from an estimation based on the knowledge of the production processes and of their accuracy.')),
                ('vertical_geometry_estimated_accuracy', models.CharField(blank=True, help_text='Value of estimatedAccuracy shall be given in meters', max_length=1)),
                ('height_below_ground', models.FloatField(blank=True, null=True)),
                ('number_of_floors_below_ground', models.SmallIntegerField(blank=True, help_text='Number of floors below ground.', null=True)),
                ('is_building_part', models.BooleanField(default=False, help_text='Is it a BuildingPart?')),
            ],
            options={
                'verbose_name': 'Building',
                'verbose_name_plural': 'Buildings',
            },
        ),
        migrations.CreateModel(
            name='CadastralParcel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('local_id', models.CharField(help_text='A local identifier, assigned by the data provider. The local identifier is unique within the namespace, that is no other spatial object carries the same unique identifier.<br />NOTE It is the responsibility of the data provider to guarantee uniqueness of the local identifier within the namespace.', max_length=32)),
                ('version_id', models.CharField(blank=True, help_text='The identifier of the particular version of the spatial object, with a maximum length of 25 characters. If the specification of a spatial object type with an external object identifier includes life-cycle information, the version identifier is used to distinguish between the different versions of a spatial object. Within the set of all versions of a spatial object, the version identifier is unique.<br />NOTE The maximum length has been selected to allow for time stamps based on ISO 8601, for example, "2007-02-12T12:12:12+05:30" as the version identifier.<br />NOTE 2 The property is void, if the spatial data set does not distinguish between different versions of the spatial object. It is missing, if the spatial object type does not support any life-cycle information.', max_length=25)),
                ('begin_lifespan_version', models.DateTimeField(help_text='Registered area value giving quantification of the area projected on the horizontal plane of the cadastral parcel.')),
                ('end_lifespan_version', models.DateTimeField(blank=True, help_text='Date and time at which this version of the spatial object was superseded or retired in the spatial data set.', null=True)),
                ('label', models.CharField(blank=True, help_text='Text commonly used to display the cadastral parcel identification.', max_length=32)),
                ('national_cadastal_reference', models.CharField(db_index=True, help_text='Thematic identifier at national level, generally the full national code of the cadastral parcel. Must ensure the link to the national cadastral register or equivalent.', max_length=32)),
                ('area_value', models.IntegerField(blank=True, help_text='Registered area value giving quantification of the area projected on the horizontal plane of the cadastral parcel.', null=True)),
                ('valid_from', models.DateTimeField(blank=True, help_text='Official date and time the cadastral parcel was/will be legally established.', null=True)),
                ('valid_to', models.DateTimeField(blank=True, help_text='Date and time at which the cadastral parcel legally ceased/will cease to be used.', null=True)),
                ('geometry', django.contrib.gis.db.models.fields.MultiPolygonField(blank=True, help_text='Geometry of the cadastral parcel.', null=True, srid=4326)),
                ('reference_point', django.contrib.gis.db.models.fields.PointField(blank=True, help_text='A point within the cadastral parcel. EXAMPLE The centroid of the cadastral parcel geometry.', null=True, srid=4326)),
            ],
            options={
                'verbose_name': 'Cadastral Parcel',
                'verbose_name_plural': 'Cadastral Parcels',
            },
        ),
        migrations.CreateModel(
            name='CodeList',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(db_index=True, max_length=64)),
                ('slug', models.CharField(blank=True, db_index=True, max_length=64)),
                ('link', models.URLField(blank=True)),
                ('label', models.CharField(max_length=128)),
                ('definition', models.TextField(blank=True)),
                ('description', models.TextField(blank=True)),
                ('application_schema', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.ApplicationSchema')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.CodeList')),
            ],
            options={
                'verbose_name': 'Code list',
                'verbose_name_plural': 'Code lists',
                'ordering': ['code', 'label'],
            },
        ),
        migrations.CreateModel(
            name='CodeListValue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(db_index=True, max_length=96)),
                ('slug', models.CharField(blank=True, db_index=True, max_length=96)),
                ('link', models.URLField()),
                ('label', models.CharField(max_length=200)),
                ('definition', models.TextField(blank=True)),
                ('description', models.TextField(blank=True)),
                ('code_list', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.CodeList')),
            ],
            options={
                'verbose_name': 'Code list value',
                'verbose_name_plural': 'Code list values',
                'ordering': ['code_list__code', 'code', 'label'],
            },
        ),
        migrations.CreateModel(
            name='Namespace',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(help_text='Namespace', max_length=32)),
                ('name', models.CharField(blank=True, max_length=64)),
            ],
            options={
                'verbose_name': 'Namespace',
                'verbose_name_plural': 'Namespaces',
                'ordering': ['code', 'name'],
            },
        ),
        migrations.CreateModel(
            name='Status',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(db_index=True, max_length=32)),
                ('slug', models.CharField(blank=True, db_index=True, max_length=32)),
                ('label', models.CharField(max_length=128)),
                ('definition', models.TextField(blank=True)),
                ('link', models.URLField()),
                ('is_valid', models.BooleanField(default=True)),
            ],
            options={
                'verbose_name': 'Status',
                'verbose_name_plural': 'Status',
                'ordering': ['is_valid', 'label'],
            },
        ),
        migrations.CreateModel(
            name='UnitOfMeasure',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, help_text='The name(s) of a particular unit of measure. Examples would include the following: 1) for uomArea - square feet <br />2) for uomTime - seconds <br />3) for uomArea - miles<br />4) uomAngle - degrees.', max_length=32)),
                ('slug', models.CharField(blank=True, db_index=True, max_length=32)),
                ('symbol', models.CharField(help_text='The symbol used for this unit of measure, such at "ft" for feet, or "m" for meter.', max_length=8)),
                ('measure_type', models.CharField(choices=[('', 'Unknown'), ('area', 'Area'), ('length', 'Length'), ('angle', 'Angle'), ('time', 'Time'), ('velocity', 'Velocity'), ('volume', 'Volume'), ('scale', 'Scale'), ('weight', 'Weight')], help_text='Measure Type', max_length=8)),
                ('name_standard_unit', models.CharField(blank=True, help_text='Name of the standard units to which this unit of measure can be directly converted. If this variable is NULL, then the standard unit for this measure type given by the local copy of the StandardsUnits code list.', max_length=8)),
                ('scale_to_standard_unit', models.FloatField(blank=True, help_text='If the implementation system used for this object does not support NULL, the  scale set to 0 is equivalent to NULL for both scale and offset.<br />If X is the current unit, and S is the standard one the of two variables scale(ToStandardUnit) and offset(ToStandardUnit) can be used to make the conversion from X to S by:<br />S = offset + scale*X <br />and, conversely, <br />X = (S-offset)/scale', null=True)),
                ('offset_to_standard_unit', models.FloatField(blank=True, help_text='See scaleToStandardUnit for a description. Again, this variable is NULL is no linear conversion is possible. If the two units are only a scale in difference, then this number is zero (0). If the implementation system used for this object does not support NULL, the then scale set to 0 is equivalent to NULL for both scale and offset.', null=True)),
                ('formula', models.CharField(blank=True, help_text='An algebraic formula (probably in some programming language) converting this unit of measure (represented in the formula by its uomSymbol) to the ISO standard (represented by its symbol. This member attribute is not required, but it is a valuable piece of documentation. ', max_length=32)),
            ],
            options={
                'verbose_name': 'Unit Of Measure',
                'verbose_name_plural': 'Units Of Measure',
                'ordering': ['symbol', 'name'],
            },
        ),
        migrations.CreateModel(
            name='Theme',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(db_index=True, max_length=32)),
                ('slug', models.CharField(blank=True, db_index=True, max_length=32)),
                ('link', models.URLField()),
                ('version', models.SmallIntegerField(blank=True, default=0)),
                ('label', models.CharField(max_length=128)),
                ('definition', models.TextField(blank=True)),
                ('description', models.TextField(blank=True)),
                ('status', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.Status')),
            ],
            options={
                'verbose_name': 'Theme',
                'verbose_name_plural': 'Themes',
                'ordering': ['code', 'label'],
            },
        ),
        migrations.CreateModel(
            name='OtherConstruction',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('local_id', models.CharField(help_text='A local identifier, assigned by the data provider. The local identifier is unique within the namespace, that is no other spatial object carries the same unique identifier.<br />NOTE It is the responsibility of the data provider to guarantee uniqueness of the local identifier within the namespace.', max_length=32)),
                ('version_id', models.CharField(blank=True, help_text='The identifier of the particular version of the spatial object, with a maximum length of 25 characters. If the specification of a spatial object type with an external object identifier includes life-cycle information, the version identifier is used to distinguish between the different versions of a spatial object. Within the set of all versions of a spatial object, the version identifier is unique.<br />NOTE The maximum length has been selected to allow for time stamps based on ISO 8601, for example, "2007-02-12T12:12:12+05:30" as the version identifier.<br />NOTE 2 The property is void, if the spatial data set does not distinguish between different versions of the spatial object. It is missing, if the spatial object type does not support any life-cycle information.', max_length=25)),
                ('begin_lifespan_version', models.DateTimeField(help_text='Registered area value giving quantification of the area projected on the horizontal plane of the cadastral parcel.')),
                ('end_lifespan_version', models.DateTimeField(blank=True, help_text='Date and time at which this version of the spatial object was superseded or retired in the spatial data set.', null=True)),
                ('date_of_construction_beginning', models.DateTimeField(blank=True, help_text='Beginning date of construction', null=True)),
                ('date_of_construction_end', models.DateTimeField(blank=True, help_text='End of date of construction', null=True)),
                ('date_of_construction_any_point', models.DateTimeField(blank=True, help_text='Any point of date of construction', null=True)),
                ('date_of_demolition_beginning', models.DateTimeField(blank=True, help_text='Beginning date of demolition', null=True)),
                ('date_of_demolition_end', models.DateTimeField(blank=True, help_text='End of date of demolition', null=True)),
                ('date_of_demolition_any_point', models.DateTimeField(blank=True, help_text='Any point of date of demolition', null=True)),
                ('date_of_renovation_beginning', models.DateTimeField(blank=True, help_text='Beginning date of last major renovation', null=True)),
                ('date_of_renovation_end', models.DateTimeField(blank=True, help_text='End of date of last major renovation', null=True)),
                ('date_of_renovation_any_point', models.DateTimeField(blank=True, help_text='Any point of date of last major renovation', null=True)),
                ('geometry', django.contrib.gis.db.models.fields.MultiPolygonField(blank=True, help_text='2D or 2.5D geometric representation', null=True, srid=4326)),
                ('building', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.Building')),
                ('cadastral_parcels', models.ManyToManyField(to='inspire_eu.CadastralParcel')),
                ('condition_of_construction', models.ForeignKey(help_text='Status of the construction.<br />EXAMPLES: functional, projected, ruin', limit_choices_to={'code_list__code': 'ConditionOfConstructionValue'}, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_otherconstruction_condition_of_construction', related_query_name='inspire_eu_otherconstruction_condition_of_constructions', to='inspire_eu.CodeListValue')),
                ('namespace', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_otherconstruction_namespace', related_query_name='inspire_eu_otherconstruction_namespaces', to='inspire_eu.Namespace')),
                ('other_construction_nature', models.ForeignKey(blank=True, help_text='Characteristic of the building that makes it generally of interest for mappings applications. The characteristic may be related to the physical aspect and/or to the function of the building.<br />This attribute focuses on the physical aspect of the building; however, this physical aspect is often expressed as a function (e.g. stadium, silo, windmill); this attribute aims to fulfil mainly mapping purposes and addresses only specific, noticeable buildings. ', limit_choices_to={'code_list__code': 'OtherConstructionNatureValue'}, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_otherconstruction_other_construction_nature', related_query_name='inspire_eu_otherconstruction_other_construction_natures', to='inspire_eu.CodeListValue')),
            ],
            options={
                'verbose_name': 'Other Construction',
                'verbose_name_plural': 'Other Constructions',
            },
        ),
        migrations.AddField(
            model_name='codelistvalue',
            name='status',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.Status'),
        ),
        migrations.AddField(
            model_name='codelist',
            name='status',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.Status'),
        ),
        migrations.AddField(
            model_name='codelist',
            name='themes',
            field=models.ManyToManyField(to='inspire_eu.Theme'),
        ),
        migrations.CreateModel(
            name='CadastralZoning',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('local_id', models.CharField(help_text='A local identifier, assigned by the data provider. The local identifier is unique within the namespace, that is no other spatial object carries the same unique identifier.<br />NOTE It is the responsibility of the data provider to guarantee uniqueness of the local identifier within the namespace.', max_length=32)),
                ('version_id', models.CharField(blank=True, help_text='The identifier of the particular version of the spatial object, with a maximum length of 25 characters. If the specification of a spatial object type with an external object identifier includes life-cycle information, the version identifier is used to distinguish between the different versions of a spatial object. Within the set of all versions of a spatial object, the version identifier is unique.<br />NOTE The maximum length has been selected to allow for time stamps based on ISO 8601, for example, "2007-02-12T12:12:12+05:30" as the version identifier.<br />NOTE 2 The property is void, if the spatial data set does not distinguish between different versions of the spatial object. It is missing, if the spatial object type does not support any life-cycle information.', max_length=25)),
                ('begin_lifespan_version', models.DateTimeField(help_text='Registered area value giving quantification of the area projected on the horizontal plane of the cadastral parcel.')),
                ('end_lifespan_version', models.DateTimeField(blank=True, help_text='Date and time at which this version of the spatial object was superseded or retired in the spatial data set.', null=True)),
                ('estimated_accuracy', models.FloatField(blank=True, help_text='The estimated absolute positional accuracy of cadastral parcels within the cadastral zoning in the used INSPIRE coordinate reference system. Absolute positional accuracy is the mean value of the positional uncertainties for a set of positions, where the positional uncertainties are the distance between a measured position and what is considered as the corresponding true position.')),
                ('geometry', django.contrib.gis.db.models.fields.MultiPolygonField(blank=True, help_text='Geometry of the cadastral zoning.', null=True, srid=4326)),
                ('label', models.CharField(blank=True, help_text='Text commonly used to display the cadastral zoning identification.', max_length=32)),
                ('name', models.CharField(blank=True, help_text='Name of the cadastral zoning.', max_length=24)),
                ('national_cadastal_zoning_reference', models.CharField(blank=True, help_text='Text commonly used to display the cadastral zoning identification.', max_length=32)),
                ('area_value', models.IntegerField(blank=True, help_text='Registered area value giving quantification of the area projected on the horizontal plane of the cadastral parcel.', null=True)),
                ('original_map_scale_denominator', models.SmallIntegerField(blank=True, help_text='The denominator in the scale of the original paper map (if any) to whose extent the cadastral zoning corresponds.', null=True)),
                ('valid_from', models.DateTimeField(blank=True, help_text='Official date and time the cadastral zoning was/will be legally established.', null=True)),
                ('valid_to', models.DateTimeField(blank=True, help_text='Date and time at which the cadastral zoning legally ceased/will cease to be used.', null=True)),
                ('reference_point', django.contrib.gis.db.models.fields.PointField(blank=True, help_text='A point within the cadastral zoning. EXAMPLE The centroid of the cadastral zoneing geometry.', null=True, srid=4326)),
                ('area_value_uom', models.ForeignKey(blank=True, limit_choices_to={'measure_type': 'area'}, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_cadastralzoning_area_value_uom', related_query_name='inspire_eu_cadastralzoning_area_value_uoms', to='inspire_eu.UnitOfMeasure')),
                ('estimated_accuracy_uom', models.ForeignKey(blank=True, help_text='Value of estimatedAccuracy shall be given in meters', limit_choices_to={'measure_type': 'area'}, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_cadastralzoning_estimated_accuracy_uom', related_query_name='inspire_eu_cadastralzoning_estimated_accuracy_uoms', to='inspire_eu.UnitOfMeasure')),
                ('level', models.ForeignKey(blank=True, help_text='Level of the cadastral zoning in the national cadastral hierarchy.', limit_choices_to={'code_list__code': 'LevelValue'}, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_cadastralzoning_level', related_query_name='inspire_eu_cadastralzoning_levels', to='inspire_eu.CodeListValue')),
                ('namespace', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_cadastralzoning_namespace', related_query_name='inspire_eu_cadastralzoning_namespaces', to='inspire_eu.Namespace')),
                ('upper_level_unit', models.ForeignKey(blank=True, help_text='Level of the cadastral zoning in the national cadastral hierarchy.', null=True, on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.CadastralZoning')),
            ],
            options={
                'verbose_name': 'Cadastral Zoning',
                'verbose_name_plural': 'Cadastral Zonings',
            },
        ),
        migrations.AddField(
            model_name='cadastralparcel',
            name='area_value_uom',
            field=models.ForeignKey(blank=True, limit_choices_to={'measure_type': 'area'}, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_cadastralparcel_area_value_uom', related_query_name='inspire_eu_cadastralparcel_area_value_uoms', to='inspire_eu.UnitOfMeasure'),
        ),
        migrations.AddField(
            model_name='cadastralparcel',
            name='cadastral_zoning',
            field=models.ForeignKey(blank=True, help_text='Cadastral Zoning', null=True, on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.CadastralZoning'),
        ),
        migrations.AddField(
            model_name='cadastralparcel',
            name='namespace',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_cadastralparcel_namespace', related_query_name='inspire_eu_cadastralparcel_namespaces', to='inspire_eu.Namespace'),
        ),
        migrations.CreateModel(
            name='BuildingNature',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('building', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.Building')),
                ('nature', models.ForeignKey(help_text='Characteristic of the building that makes it generally of interest for mappings applications. The characteristic may be related to the physical aspect and/or to the function of the building.<br />This attribute focuses on the physical aspect of the building; however, this physical aspect is often expressed as a function (e.g. stadium, silo, windmill); this attribute aims to fulfil mainly mapping purposes and addresses only specific, noticeable buildings.', limit_choices_to={'code_list__code': 'BuildingNatureValue'}, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_buildingnature_nature', related_query_name='inspire_eu_buildingnature_natures', to='inspire_eu.CodeListValue')),
            ],
            options={
                'verbose_name': 'Building Nature',
                'verbose_name_plural': 'Building Natures',
                'ordering': ['building', 'nature'],
            },
        ),
        migrations.CreateModel(
            name='BuildingHeightAboveGround',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.SmallIntegerField(help_text='Value of the height above ground')),
                ('building', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.Building')),
                ('height_reference', models.ForeignKey(help_text='Element used as the high reference.<br />EXAMPLE: The height of the building has been captured up to the top of building.', limit_choices_to={'code_list__code': 'ElevationReferenceValue'}, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_buildingheightaboveground_height_reference', related_query_name='inspire_eu_buildingheightaboveground_height_references', to='inspire_eu.CodeListValue')),
                ('low_reference', models.ForeignKey(help_text='Element as the low reference.<br />EXAMPLE: The height of the building has been captured from its the lowest ground point.', limit_choices_to={'code_list__code': 'ElevationReferenceValue'}, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_buildingheightaboveground_low_reference', related_query_name='inspire_eu_buildingheightaboveground_low_references', to='inspire_eu.CodeListValue')),
                ('status', models.ForeignKey(help_text='The way the height has been captured.', limit_choices_to={'code_list__code': 'HeightStatusValue'}, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_buildingheightaboveground_height_status', related_query_name='inspire_eu_buildingheightaboveground_s_height_statuss', to='inspire_eu.CodeListValue')),
            ],
            options={
                'verbose_name': 'Building Height Above Ground',
                'verbose_name_plural': 'Building Heights Above Ground',
                'ordering': ['building', 'height_reference'],
            },
        ),
        migrations.CreateModel(
            name='BuildingGeographicalName',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('language', models.CharField(blank=True, help_text='Language of the name, given as a three letters code, in accordance with either ISO 639-3 or ISO 639-5.', max_length=64)),
                ('source_of_name', models.CharField(blank=True, help_text='Original data source from which the geographical name is taken from and integrated in the data set providing/publishing it. For some named spatial objects it might refer again to the publishing data set if no other information is available.', max_length=64)),
                ('pronunciation', models.CharField(blank=True, help_text='Proper, correct or standard (standard within the linguistic community concerned) pronunciation of the geographical name.', max_length=64)),
                ('spelling', models.CharField(blank=True, help_text='A proper way of writing the geographical name.', max_length=64)),
                ('building', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.Building')),
                ('grammatical_gender', models.ForeignKey(blank=True, help_text='Class of nouns reflected in the behavior of associated words.', limit_choices_to={'code_list__code': 'GrammaticalGenderValue'}, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_buildinggeographicalname_grammatical_gender', related_query_name='inspire_eu_buildinggeographicalname_grammatical_genders', to='inspire_eu.CodeListValue')),
                ('grammatical_number', models.ForeignKey(blank=True, help_text='Grammatical category of nouns that expresses count distinctions.', limit_choices_to={'code_list__code': 'GrammaticalNumberValue'}, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_buildinggeographicalname_grammatical_number', related_query_name='inspire_eu_buildinggeographicalname_grammatical_numbers', to='inspire_eu.CodeListValue')),
                ('name_status', models.ForeignKey(blank=True, help_text='Qualitative information enabling to discern which credit should be given to the name with respect to its standardization and/or its topicality.', limit_choices_to={'code_list__code': 'NameStatusValue'}, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_buildinggeographicalname_name_status', related_query_name='inspire_eu_buildinggeographicalname_name_statuss', to='inspire_eu.CodeListValue')),
                ('nativeness', models.ForeignKey(blank=True, help_text='Information enabling to acknowledge if the name is the one that is/was used in the area where the spatial object is situated at the instant when the name is/was in use.', limit_choices_to={'code_list__code': 'NativenessValue'}, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_buildinggeographicalname_nativeness', related_query_name='inspire_eu_buildinggeographicalname_nativenesss', to='inspire_eu.CodeListValue')),
            ],
            options={
                'verbose_name': 'Geographical Name',
                'verbose_name_plural': 'Geographical Names',
                'ordering': ['building', 'source_of_name'],
            },
        ),
        migrations.CreateModel(
            name='BuildingExternalReference',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('information_system', models.URLField(help_text='Uniform Resource Identifier of the external information system')),
                ('information_system_name', models.CharField(blank=True, help_text='The name of the external information systemEXAMPLES: Danish Register of Dwellings, Spanish Cadastre', max_length=32)),
                ('reference', models.CharField(help_text='Thematic identifier of the spatial object or of any piece of information related to the spatial object.<br />NOTE: This reference will act as a foreign key to implement the association between the spatial object in the INSPIRE data set and in the external information system<br />EXAMPLE: The cadastral reference of a given building in the national cadastral register.', max_length=24)),
                ('building', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.Building')),
            ],
            options={
                'verbose_name': 'Building External Reference',
                'verbose_name_plural': 'Building External References',
                'ordering': ['building', 'reference'],
            },
        ),
        migrations.CreateModel(
            name='BuildingElevation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.FloatField(blank=True, help_text='Vertically-constrained dimensional property consisting of an absolute measure referenced to a well-defined surface which is commonly taken as origin (geoïd, water level, etc.).<br />Source: adapted from the definition given in the data specification of the theme Elevation.')),
                ('coordinate', django.contrib.gis.db.models.fields.PointField(blank=True, null=True, srid=4326)),
                ('dimension', models.IntegerField(blank=True, null=True)),
                ('building', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.Building')),
                ('reference', models.ForeignKey(help_text='Element where the elevation was measured.', limit_choices_to={'code_list__code': 'ElevationReferenceValue'}, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_buildingelevation_elevation_reference', related_query_name='inspire_eu_buildingelevation_elevation_references', to='inspire_eu.CodeListValue')),
                ('value_uom', models.ForeignKey(blank=True, help_text='Value of Elevation shall be given in meters', on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_buildingelevation_value_uom', related_query_name='inspire_eu_buildingelevation_value_uoms', to='inspire_eu.UnitOfMeasure')),
            ],
            options={
                'verbose_name': 'Building Elevation',
                'verbose_name_plural': 'Building Elevations',
                'ordering': ['building', 'dimension'],
            },
        ),
        migrations.CreateModel(
            name='BuildingDocument',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('document_link', models.URLField(help_text='The Internet address where the document may be found.')),
                ('date', models.DateTimeField(blank=True, help_text='Date of validity of the document', null=True)),
                ('document_description', models.TextField(blank=True, help_text='A short text providing overview of the document content. May be just title of the document')),
                ('building', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.Building')),
                ('source_status', models.ForeignKey(help_text='The status of the document, i.e. this attribute indicates if the document comes from official source or not', limit_choices_to={'code_list__code': 'SourceStatusValue'}, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_buildingdocument_source_status', related_query_name='inspire_eu_buildingdocument_source_statuss', to='inspire_eu.CodeListValue')),
            ],
            options={
                'verbose_name': 'Building Document',
                'verbose_name_plural': 'Building Documents',
                'ordering': ['building', 'document_link'],
            },
        ),
        migrations.CreateModel(
            name='BuildingCurrentUse',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('percentage', models.SmallIntegerField(blank=True, help_text='The proportion of the real world object, given as a percentage, devoted to this current use. <br />NOTE: The percentage of use is generally the percentage of floor area dedicated to this given use. If it is not the case, it is recommended to explain what the percentage refers to in metadata (template for additional information)', null=True)),
                ('building', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.Building')),
                ('current_use', models.ForeignKey(help_text='Current use', limit_choices_to={'code_list__code': 'CurrentUseValue'}, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_buildingcurrentuse_current_use', related_query_name='inspire_eu_buildingcurrentuse_current_uses', to='inspire_eu.CodeListValue')),
            ],
            options={
                'verbose_name': 'Building Current Use',
                'verbose_name_plural': 'Building Current Uses',
                'ordering': ['building', 'current_use'],
            },
        ),
        migrations.AddField(
            model_name='building',
            name='cadastral_parcels',
            field=models.ManyToManyField(to='inspire_eu.CadastralParcel'),
        ),
        migrations.AddField(
            model_name='building',
            name='condition_of_construction',
            field=models.ForeignKey(help_text='Status of the construction.<br />EXAMPLES: functional, projected, ruin', limit_choices_to={'code_list__code': 'ConditionOfConstructionValue'}, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_building_condition_of_construction', related_query_name='inspire_eu_building_condition_of_constructions', to='inspire_eu.CodeListValue'),
        ),
        migrations.AddField(
            model_name='building',
            name='height_below_ground_uom',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_building_height_below_ground_uom', related_query_name='inspire_eu_building_height_below_ground_uoms', to='inspire_eu.UnitOfMeasure'),
        ),
        migrations.AddField(
            model_name='building',
            name='horizontal_geometry_estimated_accuracy_uom',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_building_horizontal_geometry_estimated_accuracy_uom', related_query_name='inspire_eu_building_horizontal_geometry_estimated_accuracy_uoms', to='inspire_eu.UnitOfMeasure'),
        ),
        migrations.AddField(
            model_name='building',
            name='horizontal_geometry_reference',
            field=models.ForeignKey(help_text='Element of the building that was captured by (X,Y) coordinates.', limit_choices_to={'code_list__code': 'HorizontalGeometryReferenceValue'}, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_building_horizontal_geometry_reference', related_query_name='inspire_eu_building_horizontal_geometry_references', to='inspire_eu.CodeListValue'),
        ),
        migrations.AddField(
            model_name='building',
            name='namespace',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_building_namespace', related_query_name='inspire_eu_building_namespaces', to='inspire_eu.Namespace'),
        ),
        migrations.AddField(
            model_name='building',
            name='parent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='parts', to='inspire_eu.Building'),
        ),
        migrations.AddField(
            model_name='building',
            name='vertical_geometry_estimated_accuracy_uom',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_building_vertical_geometry_estimated_accuracy_uom', related_query_name='inspire_eu_building_vertical_geometry_estimated_accuracy_uoms', to='inspire_eu.UnitOfMeasure'),
        ),
        migrations.AddField(
            model_name='building',
            name='vertical_geometry_reference',
            field=models.ForeignKey(blank=True, help_text='Element of the building that was captured by vertical coordinates.', limit_choices_to={'code_list__code': 'ElevationReferenceValue'}, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='inspire_eu_building_vertical_geometry_reference', related_query_name='inspire_eu_building_vertical_geometry_references', to='inspire_eu.CodeListValue'),
        ),
        migrations.AddField(
            model_name='applicationschema',
            name='status',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='inspire_eu.Status'),
        ),
        migrations.AddField(
            model_name='applicationschema',
            name='themes',
            field=models.ManyToManyField(to='inspire_eu.Theme'),
        ),
    ]



"""*****************
Cadastral Parcels
*****************

Definition
==========
Areas defined by cadastral registers or equivalent.

Description
===========

The INSPIRE Directive focuses on the geographical part of cadastral data. In the INSPIRE context, cadastral
parcels will be mainly used as locators for geo-information in general, including environmental data.
As much as possible, in the INSPIRE context, cadastral parcels should be forming a partition of national
territory. Cadastral parcel should be considered as a single area of Earth surface, under homogeneous real
property rights and unique ownership (adapted from UN ECE 2004 and WG-CPI, 2006). Remark: By unique ownership
is meant that the ownership is held by one or several owners for the whole parcel. By homogeneous property
rights is meant that rights of ownership, leases and mortgages affect the whole parcel. This does not apply
to specific rights as servitudes which may only affect part of the parcel.

In the definition given by the INSPIRE directive, or equivalent refers to all public agencies and institutions
other than the main traditional/nominal cadastre or land registry, that register parts of the Earth's surface
such as special domains, urban cadastres, public lands, which spatially complement the registrations by the
main cadastre or land registry.

Cadastral parcels are considered in the INSPIRE scope if they are available as vector data.

Rights and owners are out of the INSPIRE scope.

Buildings, land use, addresses are considered in other INSPIRE themes.

References
==========
    * `Data Specifications <https://inspire.ec.europa.eu/Themes/122/2892>`_
    * `INSPIRE Data Specification on Cadastral Parcels – Technical Guidelines <https://inspire.ec.europa.eu/id/document/tg/cp>`_
    * `Theme description from Registry  <https://inspire.ec.europa.eu/theme/cp>`_
    * `UML <https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:1:3:1:7204>`_

    .. figure:: ../../../docs/_static/img/cadastral_parcels.png
        :align: center
        :target: https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:1:3:1:7204

Models
======

.. automodule:: inspire_eu.models.cadastral_parcels.abstract
   :members:
   :inherited-members:
   :exclude-members: __init__,clean,clean_fields,full_clean,get_deferred_fields,refresh_from_db,save,save_base,serializable_value,validate_unique

"""  # noqa

import logging

from django.contrib.gis.db import models
from django.utils.translation import ugettext_lazy as _

from .. import INSPIRE_EU_THEMES

log = logging.getLogger(__name__)


if "cadastral_parcels" in INSPIRE_EU_THEMES and INSPIRE_EU_THEMES["cadastral_parcels"]:
    from ...models import BaseInspireEUModel
    from .abstract import AbstractCadastralParcel, AbstractCadastralZoning

    class CadastralZoning(BaseInspireEUModel, AbstractCadastralZoning, models.Model):
        """CadastralZoning

        Definition
            Intermediary areas used in order to divide national territory into cadastral parcels.

        Description
            Cadastral zonings are the intermediary areas (such as municipalities, sections, blocks, …) used in
            order to divide national territory into cadastral parcels. In the INSPIRE context, cadastral zonings are
            to be used to carry metadata information and to facilitate portrayal and search of data.
            Cadastral zonings have the following additional attributes:
            − a geometry
            − a national cadastral zoning reference
            − a name, if any
            − a level in the national cadastral hierarchy and the name of this level
            − portrayal attributes: reference point and label
            − metadata attributes: original map scale denominator and estimated accuracy
            If cadastral zonings are provided, cadastral parcels shall belong to one cadastral zoning of lowest
            level. When several levels of zonings exist in a Member State, it must be ensured that the higher level
            units are composed of that of lower level.

        References
            * `UML <https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:1:3:1:7204>`_
        """

        class Meta:
            verbose_name = _("Cadastral Zoning")
            verbose_name_plural = _("Cadastral Zonings")

        def __str__(self):
            return "%s %s" % (self.label, self.national_cadastal_zoning_reference)

    class CadastralParcel(BaseInspireEUModel, AbstractCadastralParcel, models.Model):
        """Cadastral Parcel

        Definition
            Areas defined by cadastral registers or equivalent.

        Description
            Areas defined by cadastral registers or equivalent

            NOTE As much as possible, in the INSPIRE context, cadastral parcels should be forming a partition of national
            territory. Cadastral parcel should be considered as a single area of Earth surface (land and/or water), under
            homogeneous real property rights and unique ownership, real property rights and ownership being defined by
            national law (adapted from UN ECE 2004 and WG-CPI, 2006). By unique ownership is meant that the ownership is
            held by one or several joint owners for the whole parcel.

        References
            * https://inspire.ec.europa.eu/id/document/tg/cp
            * https://inspire.ec.europa.eu/schemas/cp/4.0/CadastralParcels.xsd
            * https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:1:3:1:7209
            * https://inspire.ec.europa.eu/documents/Data_Specifications/INSPIRE_DataSpecification_CP_v3.0.1.pdf
        """  # noqa

        cadastral_zoning = models.ForeignKey(
            CadastralZoning,
            on_delete=models.PROTECT,
            blank=True,
            null=True,
            help_text=_(
                _("Cadastral Zoning"),
            ),
        )

        class Meta:
            verbose_name = _("Cadastral Parcel")
            verbose_name_plural = _("Cadastral Parcels")

        def __str__(self):
            return "%s %s" % (self.label, self.national_cadastral_reference)



import logging

from django.contrib.gis.db import models
from django.utils.translation import ugettext_lazy as _

from ...models import INSPIRE_EU_DEFAULT_SRID, CodeListValue, UnitOfMeasure
from ...models.abstract import DataLifeCycleInfo, Identifier

log = logging.getLogger(__name__)


##############################################
# Building Base
##############################################
class AbstractConstruction(Identifier, DataLifeCycleInfo, models.Model):
    """Abstract construction

    Definition:
        Abstract spatial object type grouping the semantic properties of  buildings, building parts and of
        some optional spatial object types that may be added in order to provide more information about the
        theme Buildings.

    Description:
        The optional spatial object types that may be added to core profiles are described in the extended profiles.
        The ones inheriting from the attributes of AbstractConstruction are Installation and OtherConstruction

    References
        https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:3:2:1:7895

    """

    condition_of_construction = models.ForeignKey(
        CodeListValue,
        on_delete=models.PROTECT,
        limit_choices_to={"code_list__code": "ConditionOfConstructionValue"},
        related_name="%(app_label)s_%(class)s_condition_of_construction",
        related_query_name="%(app_label)s_%(class)s_condition_of_constructions",
        help_text=_(
            "Status of the construction.<br />EXAMPLES: functional, projected, ruin",
        ),
    )

    date_of_construction_beginning = models.DateTimeField(
        blank=True,
        null=True,
        help_text=_("Beginning date of construction"),
    )
    date_of_construction_end = models.DateTimeField(
        blank=True,
        null=True,
        help_text=_("End of date of construction"),
    )
    date_of_construction_any_point = models.DateTimeField(
        blank=True,
        null=True,
        help_text=_("Any point of date of construction"),
    )

    date_of_demolition_beginning = models.DateTimeField(
        blank=True,
        null=True,
        help_text=_("Beginning date of demolition"),
    )
    date_of_demolition_end = models.DateTimeField(
        blank=True,
        null=True,
        help_text=_("End of date of demolition"),
    )
    date_of_demolition_any_point = models.DateTimeField(
        blank=True,
        null=True,
        help_text=_("Any point of date of demolition"),
    )

    date_of_renovation_beginning = models.DateTimeField(
        blank=True,
        null=True,
        help_text=_("Beginning date of last major renovation"),
    )
    date_of_renovation_end = models.DateTimeField(
        blank=True,
        null=True,
        help_text=_("End of date of last major renovation"),
    )
    date_of_renovation_any_point = models.DateTimeField(
        blank=True,
        null=True,
        help_text=_("Any point of date of last major renovation"),
    )

    class Meta:
        abstract = True


class AbstractBuilding(models.Model):
    """Abstract building

    Definition
        Abstract spatial object type grouping the common semantic properties of the spatial object types Building
        and BuildingPart.

    References
        https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:3:2:1:7900

    """

    number_of_dwellings = models.SmallIntegerField(
        blank=True,
        null=True,
        help_text=_(
            "A dwelling is a residential unit which may consist of one or several rooms designed for the occupation "
            "of households.<br />"
            "NOTE: In the data sets including building units, a dwelling is a residential building unit or, only when "
            "that building has no building units, a residential building<br />"
            "EXAMPLES: a single building dwelling could be a detached or semi-detached house. A block of flats will "
            "contain multiple dwellings determined by the number of individual flats.",
        ),
    )
    number_of_building_units = models.SmallIntegerField(
        blank=True,
        null=True,
        help_text=_(
            "Number of building units in the building. A BuildingUnit is a subdivision of Building with its own "
            "lockable access from the outside or from a common area (i.e. not from another BuildingUnit), which is "
            "atomic, functionally independent, and may be separately sold, rented out, inherited, etc.<br />"
            "Building units are spatial objects aimed at subdividing buildings and/or building parts into smaller "
            "parts that are treated as separate entities in daily life. A building unit is homogeneous, regarding "
            "management aspects.<br />"
            "EXAMPLES: It may be e.g. an apartment in a condominium, a terraced house, or a shop inside a "
            "shopping arcade.<br />"
            "NOTE 1: According to national regulations, a building unit may be a flat, a cellar, a garage or "
            "set of a flat, a cellar and a garage.<br />"
            "NOTE 2: According to national regulation, a building that is one entity for daily life (typically, "
            "a single family house) may be considered as a Building composed of one BuildingUnit or as a Building "
            "composed of zero BuildingUnit.",
        ),
    )
    number_of_floors_above_ground = models.SmallIntegerField(
        blank=True,
        null=True,
        help_text=_("Number of floors above ground."),
    )

    class Meta:
        abstract = True


class AbstractBuildingGeometry2D(models.Model):
    """Building geometry 2D

    Definition
        This data types includes the geometry of the building and metadata information about which element of
        the building was captured and how.

    References
        https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:3:2:1:7904

    """

    geometry = models.MultiPolygonField(
        srid=INSPIRE_EU_DEFAULT_SRID,
        blank=True,
        null=True,
        help_text=_("2D or 2.5D geometric representation"),
    )
    reference_geometry = models.BooleanField(
        help_text=_(
            "The geometry to be taken into account by view services, for portrayal.<br />"
            "NOTE 1: In case of multiple representation by point and by surface, it is generally recommended to "
            "provide the surface as reference geometry.<br />"
            "NOTE 2: The geometric representation whose referenceGeometry is true may also be used preferably "
            "for spatial queries by download services (WFS) or by Geographical Information System (GIS).",
        ),
    )
    # HorizontalGeometryReferenceValue
    horizontal_geometry_reference = models.ForeignKey(
        CodeListValue,
        on_delete=models.PROTECT,
        limit_choices_to={"code_list__code": "HorizontalGeometryReferenceValue"},
        related_name="%(app_label)s_%(class)s_horizontal_geometry_reference",
        related_query_name="%(app_label)s_%(class)s_horizontal_geometry_references",
        help_text=_("Element of the building that was captured by (X,Y) coordinates."),
    )

    horizontal_geometry_estimated_accuracy = models.FloatField(
        blank=True,
        help_text=_(
            "The estimated absolute positional accuracy of the (X,Y) coordinates of the building geometry, "
            "in the INSPIRE official Coordinate Reference System. Absolute positional accuracy is defined "
            "as the mean value of the positional uncertainties for a set of positions where the positional "
            "uncertainties are defined as the distance between a measured position and what is considered as "
            "the corresponding true position.<br />"
            "NOTE: This mean value may come from quality measures on a homogeneous population of buildings or "
            "from an estimation based on the knowledge of the production processes and of their accuracy.",
        ),
    )
    horizontal_geometry_estimated_accuracy_uom = models.ForeignKey(
        UnitOfMeasure,
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        related_name="%(app_label)s_%(class)s_horizontal_geometry_estimated_accuracy_uom",
        related_query_name="%(app_label)s_%(class)s_horizontal_geometry_estimated_accuracy_uoms",
    )

    vertical_geometry_estimated_accuracy = models.CharField(
        max_length=1,
        blank=True,
        help_text=_("Value of estimatedAccuracy shall be given in meters"),
    )
    vertical_geometry_estimated_accuracy_uom = models.ForeignKey(
        UnitOfMeasure,
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        related_name="%(app_label)s_%(class)s_vertical_geometry_estimated_accuracy_uom",
        related_query_name="%(app_label)s_%(class)s_vertical_geometry_estimated_accuracy_uoms",
    )
    # ElevationReferenceValue
    vertical_geometry_reference = models.ForeignKey(
        CodeListValue,
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        limit_choices_to={"code_list__code": "ElevationReferenceValue"},
        related_name="%(app_label)s_%(class)s_vertical_geometry_reference",
        related_query_name="%(app_label)s_%(class)s_vertical_geometry_references",
        help_text=_(
            "Element of the building that was captured by vertical coordinates.",
        ),
    )

    class Meta:
        abstract = True


##############################################
# Building Extended
##############################################
class AbstractBuildingExtended2D(AbstractBuilding):
    """BuildingsExtended2D

    Definition
        The BuildingsExtended2D profile is a semantic extension of Buildings2D profile with
        additional thematic attributes (material of construction, official area or value, connection to
        utility networks...), classes (building units, installations, other constructions) and references to
        other data (like cadastral data and addresses).

    References
        https://inspire.ec.europa.eu/documents/Data_Specifications/INSPIRE_DataSpecification_BU_v3.0.pdf
    """

    # energy_performance = models.ForeignKey(EnergyPerformance, blank=True, null=True, on_delete=models.PROTECT)
    height_below_ground = models.FloatField(blank=True, null=True)
    height_below_ground_uom = models.ForeignKey(
        UnitOfMeasure,
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        related_name="%(app_label)s_%(class)s_height_below_ground_uom",
        related_query_name="%(app_label)s_%(class)s_height_below_ground_uoms",
    )
    number_of_floors_below_ground = models.SmallIntegerField(
        blank=True,
        null=True,
        help_text=_("Number of floors below ground."),
    )

    class Meta:
        abstract = True


class AbstractOtherConstruction(AbstractConstruction):
    """Abstract other construction

    Definition
        Abstract spatial object type grouping the semantic properties of other
        constructions. An other construction is a self-standing construction that belongs
        to theme Buildings and that is not a Building

    Description

        NOTE 1: the main difference between a building and an other construction is the fact that an other construction
        does not need to be enclosed.

        NOTE 2: the other constructions to be considered under scope of theme Buildings are the constructions that are
        not present in another INSPIRE theme and that are necessary for environmental use cases, such as the ones
        considered in this data specification.

        EXAMPLES: bridge, acoustic fence, city wall.

    """

    # OtherConstructionNatureValue
    other_construction_nature = models.ForeignKey(
        CodeListValue,
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        limit_choices_to={"code_list__code": "OtherConstructionNatureValue"},
        related_name="%(app_label)s_%(class)s_other_construction_nature",
        related_query_name="%(app_label)s_%(class)s_other_construction_natures",
        help_text=_(
            "Characteristic of the building that makes it generally of interest for mappings applications. "
            "The characteristic may be related to the physical aspect and/or to the function of the building.<br />"
            "This attribute focuses on the physical aspect of the building; however, this physical aspect is often "
            "expressed as a function (e.g. stadium, silo, windmill); this attribute aims to fulfil mainly mapping "
            "purposes and addresses only specific, noticeable buildings. ",
        ),
    )

    class Meta:
        abstract = True


class AbstractDocument(models.Model):
    """Document

    Definition
        Any document providing information about the building or building part or building unit.


    Description
        EXAMPLES: the building permit, a photo of facade or inner yard, a sketch of interior, the building code, the
                energy performance assessment, an emergency plan
    """

    document_link = models.URLField(
        help_text=_("The Internet address where the document may be found."),
    )
    date = models.DateTimeField(
        blank=True,
        null=True,
        help_text=_("Date of validity of the document"),
    )
    document_description = models.TextField(
        blank=True,
        help_text=_(
            "A short text providing overview of the document content. May be just title of the document",
        ),
    )
    # SourceStatusValue
    source_status = models.ForeignKey(
        CodeListValue,
        limit_choices_to={"code_list__code": "SourceStatusValue"},
        related_name="%(app_label)s_%(class)s_source_status",
        related_query_name="%(app_label)s_%(class)s_source_statuss",
        on_delete=models.PROTECT,
        help_text=_(
            "The status of the document, i.e. this attribute indicates if the document comes from official "
            "source or not",
        ),
    )

    class Meta:
        abstract = True


class AbstractOfficialArea(models.Model):
    """Official area

    Definition
        This data types includes the official area of the building, building part or building unit and information
        about the exact meaning of this area.

    """

    # OfficialAreaReferenceValue
    reference = models.ForeignKey(
        CodeListValue,
        on_delete=models.PROTECT,
        limit_choices_to={"code_list__code": "OfficialAreaReferenceValue"},
        help_text=_(
            "The type of official area may be described either by using the values provided by "
            "the CLGE measurement code for the floor area of buildings (which values are "
            "provided by the CLGE_OfficialAreaReferenceValue) or by using another "
            "standard (which values are provided by the empty code list OtherStandard "
            "OfficialAreaReferenceValue, this code list having to be defined at Member State "
            "level)..",
        ),
        related_name="%(app_label)s_%(class)s_official_area_reference",
        related_query_name="%(app_label)s_%(class)s_official_area_references",
    )
    value = models.FloatField(blank=True, help_text=_("The value of the official area"))
    # value_uom = models.CharField(max_length=1,
    value_uom = models.ForeignKey(
        UnitOfMeasure,
        on_delete=models.PROTECT,
        blank=True,
        related_name="%(app_label)s_%(class)s_value_uom",
        related_query_name="%(app_label)s_%(class)s_value_uoms",
    )
    height_parameter = models.FloatField(
        blank=True,
        null=True,
        help_text=_(
            "The height parameter used to differentiate internal primary area of internal other "
            "area, if the official area is referenced using the CLGE measurement code for the "
            "floor area of buildings",
        ),
    )
    height_parameter_uom = models.ForeignKey(
        UnitOfMeasure,
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        help_text=_("Value of Elevation shall be given in meters"),
        related_name="%(app_label)s_%(class)s_height_parameter_uom",
        related_query_name="%(app_label)s_%(class)s_height_parameter_uoms",
    )

    class Meta:
        abstract = True


class AbstractElevation(models.Model):
    """Elevation

    Definition
        This data types includes the elevation value itself  and information on how this elevation was measured.


    References
        https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:3:2:1:7901
    """

    # ElevationReferenceValue
    reference = models.ForeignKey(
        CodeListValue,
        on_delete=models.PROTECT,
        limit_choices_to={"code_list__code": "ElevationReferenceValue"},
        help_text=_("Element where the elevation was measured."),
        related_name="%(app_label)s_%(class)s_elevation_reference",
        related_query_name="%(app_label)s_%(class)s_elevation_references",
    )
    value = models.FloatField(
        blank=True,
        help_text=_(
            "Vertically-constrained dimensional property consisting of an absolute measure referenced to a "
            "well-defined surface which is commonly taken as origin (geoïd, water level, etc.).<br />"
            "Source: adapted from the definition given in the data specification of the theme Elevation.",
        ),
    )
    # value_uom = models.CharField(max_length=1,
    value_uom = models.ForeignKey(
        UnitOfMeasure,
        on_delete=models.PROTECT,
        blank=True,
        help_text=_("Value of Elevation shall be given in meters"),
        related_name="%(app_label)s_%(class)s_value_uom",
        related_query_name="%(app_label)s_%(class)s_value_uoms",
    )

    class Meta:
        abstract = True


class AbstractExternalReference(models.Model):
    """External reference

    Definition
        Reference to an external information system containing any piece of information related to the spatial object.

    References
        https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:3:2:1:7908

    """

    information_system = models.URLField(
        help_text=_("Uniform Resource Identifier of the external information system"),
    )
    information_system_name = models.CharField(
        max_length=32,
        blank=True,
        help_text=_(
            "The name of the external information system"
            "EXAMPLES: Danish Register of Dwellings, Spanish Cadastre",
        ),
    )
    reference = models.CharField(
        max_length=24,
        help_text=_(
            "Thematic identifier of the spatial object or of any piece of information related to the "
            "spatial object.<br />"
            "NOTE: This reference will act as a foreign key to implement the association between the spatial "
            "object in the INSPIRE data set and in the external information system<br />"
            "EXAMPLE: The cadastral reference of a given building in the national cadastral register.",
        ),
    )

    class Meta:
        abstract = True


class AbstractHeightAboveGround(models.Model):
    """Height above ground

    Definition
        Vertical distance (measured or estimated) between a low reference and a high reference.

    References
        https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:3:2:1:7898

    """

    # ElevationReferenceValue
    height_reference = models.ForeignKey(
        CodeListValue,
        on_delete=models.PROTECT,
        limit_choices_to={"code_list__code": "ElevationReferenceValue"},
        help_text=_(
            "Element used as the high reference.<br />"
            "EXAMPLE: The height of the building has been captured up to the top of building.",
        ),
        related_name="%(app_label)s_%(class)s_height_reference",
        related_query_name="%(app_label)s_%(class)s_height_references",
    )
    # ElevationReferenceValue
    low_reference = models.ForeignKey(
        CodeListValue,
        on_delete=models.PROTECT,
        limit_choices_to={"code_list__code": "ElevationReferenceValue"},
        help_text=_(
            "Element as the low reference.<br />"
            "EXAMPLE: The height of the building has been captured from its the lowest ground point.",
        ),
        related_name="%(app_label)s_%(class)s_low_reference",
        related_query_name="%(app_label)s_%(class)s_low_references",
    )
    # HeightStatusValue
    status = models.ForeignKey(
        CodeListValue,
        on_delete=models.PROTECT,
        limit_choices_to={"code_list__code": "HeightStatusValue"},
        help_text=_("The way the height has been captured."),
        related_name="%(app_label)s_%(class)s_height_status",
        related_query_name="%(app_label)s_%(class)s_s_height_statuss",
    )
    value = models.SmallIntegerField(
        help_text=_("Value of the height above ground"),
    )

    class Meta:
        abstract = True


class AbstractBuildingNature(models.Model):
    """Building nature

    Definition
        Characteristic of the building that makes it generally of interest for mappings applications.
        The characteristic may be related to the physical aspect and/or to the function of the building.

    Description
        This attribute focuses on the physical aspect of the building; however, this physical aspect is often
        expressed as a function (e.g. stadium, silo, windmill); this attribute aims to fulfil mainly mapping
        purposes and addresses only specific, noticeable buildings.

    Vocabulary
        https://inspire.ec.europa.eu/codeList/BuildingNatureValue

    """

    # BuildingNatureValue
    nature = models.ForeignKey(
        CodeListValue,
        on_delete=models.PROTECT,
        limit_choices_to={"code_list__code": "BuildingNatureValue"},
        related_name="%(app_label)s_%(class)s_nature",
        related_query_name="%(app_label)s_%(class)s_natures",
        help_text=_(
            "Characteristic of the building that makes it generally of interest for mappings applications. The "
            "characteristic may be related to the physical aspect and/or to the function of the building.<br />"
            "This attribute focuses on the physical aspect of the building; however, this physical aspect is "
            "often expressed as a function (e.g. stadium, silo, windmill); this attribute aims to fulfil mainly "
            "mapping purposes and addresses only specific, noticeable buildings.",
        ),
    )

    class Meta:
        abstract = True


class AbstractBuildingCurrentUse(models.Model):
    """Current use

    Definition
        This data type enables to detail the current use(s).

    References
        https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:3:2:1:7906

    """

    # CurrentUseValue
    current_use = models.ForeignKey(
        CodeListValue,
        on_delete=models.PROTECT,
        limit_choices_to={"code_list__code": "CurrentUseValue"},
        related_name="%(app_label)s_%(class)s_current_use",
        related_query_name="%(app_label)s_%(class)s_current_uses",
        help_text=_("Current use"),
    )
    percentage = models.SmallIntegerField(
        blank=True,
        null=True,
        help_text=_(
            "The proportion of the real world object, given as a percentage, devoted to this current "
            "use. <br />"
            "NOTE: The percentage of use is generally the percentage of floor area dedicated to this "
            "given use. If it is not the case, it is recommended to explain what the percentage refers "
            "to in metadata (template for additional information)",
        ),
    )

    class Meta:
        abstract = True


class AbstractEnergyPerformance(models.Model):
    """Energy performance

    Definition
        The energy performance of the building or building part or building unit

    Description
        NOTE: The energy performance is required by the Energy Performance of
        Building Directive for the new buildings being rent or sold.

    """

    energy_performance_value = models.ForeignKey(
        CodeListValue,
        on_delete=models.PROTECT,
        limit_choices_to={"code_list__code": "EnergyPerformanceValue"},
        help_text=_(
            "The date when the energy performance of the building or building unit was assessed",
        ),
        related_name="%(app_label)s_%(class)s_energy_performance_value",
        related_query_name="%(app_label)s_%(class)s_energy_performance_values",
    )
    date_of_assessment = models.DateTimeField(
        blank=True,
        help_text=_(
            "Date and time at which this version of the spatial object was inserted or changed in "
            "the spatial data set.",
        ),
    )
    # DocumentCitation: INSPIRE Generic Conceptual Model, version 3.4 [DS-D2.5]
    #                   Citation for the purposes of unambiguously referencing a document.
    assessment_method = models.CharField(
        max_length=1,
        blank=True,
        help_text=_(
            "The reference to the document describing the assessment method of energy performance",
        ),
    )

    class Meta:
        abstract = True


class AbstractBuildingAndBuildingUnitInfo(models.Model):
    """Building and building unit info

    Definition
        Abstract spatial object type grouping the additional properties that are common to Building, Building Part and
        BuildingUnit.

    Description

        NOTE 1: The additional properties are those that are not already included in the base application schema.

        NOTE 2: These additional properties concern mainly the official information that may be attached to
        buildings / building parts or to building units.
    """

    connection_to_electricity = models.BooleanField(blank=True, null=True)
    connection_to_gas = models.BooleanField(blank=True, null=True)
    connection_to_sewage = models.BooleanField(blank=True, null=True)
    connection_to_water = models.BooleanField(blank=True, null=True)
    # energy_performance = models.ForeignKey(EnergyPerformance)

    class Meta:
        abstract = True


class AbstractBuildingUnit(AbstractBuildingAndBuildingUnitInfo):
    """Building unit

    Definition
        Abstract spatial object type grouping the semantic properties of building units. A BuildingUnit is a
        subdivision of Building with its own lockable access from the outside or from a common area (i.e. not
        from another BuildingUnit), which is atomic, functionally independent, and may be separately sold,
        rented out, inherited, etc.

    Description

        Building units are spatial objects aimed at subdividing buildings and/or building parts into smaller
        parts that are treated as separate entities in daily life. A building unit is homogeneous, regarding
        management aspects.

        EXAMPLES: It may be e.g. an apartment in a condominium, a terraced house, or a shop inside a shopping arcade.

        NOTE 1: According to national regulations, a building unit may be a flat, a cellar, a garage or set of a flat,
        a cellar and  a garage.

        NOTE 2: According to national regulation, a building that is one entity for daily life (typically, a single
        family house) may be considered as a Building composed of one BuildingUnit or as a Building composed of
        zero BuildingUnit.

    """

    class Meta:
        abstract = True


class AbstractBuildingInfo(AbstractBuildingAndBuildingUnitInfo):
    """Building info

    Definition
        Abstract spatial object type grouping the additional specific properties of Building and Building Part

    Description

        NOTE 1: The additional properties are those that are not already included in the base application schema.

        NOTE 2: The specific properties are the properties that apply to Building and BuildingPart without
        applying to BuildingUnit.

    .. warning::

        Pending ToDo:

        + floorDescription: FloorDescription [0..*]
        + floorDistribution: FloorRange [1..*]
        + heightBelowGround: Length [0..1]
        + materialOfFacade: MaterialOfFacadeValue [0..*]
        + materialOfRoof: MaterialOfRoofValue [0..*]
        + materialOfStructure: MaterialOfStructureValue [0..*]
        + numberOfFloorsBelowGround: Integer [0..1]
        + roofType: RoofTypeValue [0..*]
    """

    class Meta:
        abstract = True



# -*- coding: utf-8 -*-
from django.conf.urls import url
from django.views.generic import TemplateView

app_name = "inspire_eu"

urlpatterns = [
    url(r"", TemplateView.as_view(template_name="base.html")),
]



import logging

from django.contrib.gis.db import models
from django.utils.translation import ugettext_lazy as _

from ...models import INSPIRE_EU_DEFAULT_SRID, CodeListValue, UnitOfMeasure
from ...models.abstract import DataLifeCycleInfo, Identifier

log = logging.getLogger(__name__)


class AbstractCadastralZoning(Identifier, DataLifeCycleInfo):
    """CadastralZoning

    Definition
        Intermediary areas used in order to divide national territory into cadastral parcels.

    Description

        Cadastral zonings are the intermediary areas (such as municipalities, sections, blocks, …) used in
        order to divide national territory into cadastral parcels. In the INSPIRE context, cadastral zonings are
        to be used to carry metadata information and to facilitate portrayal and search of data.
        Cadastral zonings have the following additional attributes:

            * a geometry
            * a national cadastral zoning reference
            * a name, if any
            * a level in the national cadastral hierarchy and the name of this level
            * portrayal attributes: reference point and label
            * metadata attributes: original map scale denominator and estimated accuracy

        If cadastral zonings are provided, cadastral parcels shall belong to one cadastral zoning of lowest
        level. When several levels of zonings exist in a Member State, it must be ensured that the higher level
        units are composed of that of lower level.

    References:
        * `UML <https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:1:3:1:7204>`_
    """

    estimated_accuracy = models.FloatField(
        blank=True,
        help_text=_(
            "The estimated absolute positional accuracy of cadastral parcels within the cadastral zoning in the "
            "used INSPIRE coordinate reference system. Absolute positional accuracy is the mean value of the "
            "positional uncertainties for a set of positions, where the positional uncertainties are the "
            "distance between a measured position and what is considered as the corresponding true position.",
        ),
    )

    estimated_accuracy_uom = models.ForeignKey(
        UnitOfMeasure,
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        related_name="%(app_label)s_%(class)s_estimated_accuracy_uom",
        related_query_name="%(app_label)s_%(class)s_estimated_accuracy_uoms",
        limit_choices_to={"measure_type": UnitOfMeasure.MEASURE_TYPE_AREA},
        help_text=_("Value of estimatedAccuracy shall be given in meters"),
    )

    geometry = models.MultiPolygonField(
        srid=INSPIRE_EU_DEFAULT_SRID,
        blank=True,
        null=True,
        help_text=_(
            _("Geometry of the cadastral zoning."),
        ),
    )

    label = models.CharField(
        max_length=32,
        blank=True,
        help_text=_(
            "Text commonly used to display the cadastral zoning identification.",
        ),
    )

    level = models.ForeignKey(
        CodeListValue,
        blank=True,
        null=True,
        limit_choices_to={"code_list__code": "LevelValue"},
        related_name="%(app_label)s_%(class)s_level",
        related_query_name="%(app_label)s_%(class)s_levels",
        on_delete=models.PROTECT,
        help_text=_(
            "Level of the cadastral zoning in the national cadastral hierarchy.",
        ),
    )

    name = models.CharField(
        max_length=24,
        blank=True,
        help_text=_("Name of the cadastral zoning."),
    )

    national_cadastal_zoning_reference = models.CharField(
        max_length=32,
        blank=True,
        help_text=_(
            "Text commonly used to display the cadastral zoning identification.",
        ),
    )

    area_value = models.IntegerField(
        blank=True,
        null=True,
        help_text=_(
            "Registered area value giving quantification of the area projected on the horizontal plane of "
            "the cadastral parcel.",
        ),
    )
    area_value_uom = models.ForeignKey(
        UnitOfMeasure,
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        limit_choices_to={"measure_type": UnitOfMeasure.MEASURE_TYPE_AREA},
        related_name="%(app_label)s_%(class)s_area_value_uom",
        related_query_name="%(app_label)s_%(class)s_area_value_uoms",
    )

    original_map_scale_denominator = models.SmallIntegerField(
        blank=True,
        null=True,
        help_text=_(
            "The denominator in the scale of the original paper map (if any) to whose extent the cadastral "
            "zoning corresponds.",
        ),
    )

    valid_from = models.DateTimeField(
        blank=True,
        null=True,
        help_text=_(
            "Official date and time the cadastral zoning was/will be legally established.",
        ),
    )

    valid_to = models.DateTimeField(
        blank=True,
        null=True,
        help_text=_(
            "Date and time at which the cadastral zoning legally ceased/will cease to be used.",
        ),
    )

    reference_point = models.PointField(
        srid=INSPIRE_EU_DEFAULT_SRID,
        blank=True,
        null=True,
        help_text=_(
            "A point within the cadastral zoning. EXAMPLE The centroid of the cadastral zoneing geometry.",
        ),
    )

    upper_level_unit = models.ForeignKey(
        "self",
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        help_text=_(
            "Level of the cadastral zoning in the national cadastral hierarchy.",
        ),
    )

    class Meta:
        abstract = True


class AbstractCadastralParcel(
    Identifier,
    DataLifeCycleInfo,
    models.Model,
):
    """Cadastral Parcels

    Definition
        Areas defined by cadastral registers or equivalent

    Description
        The INSPIRE Directive focuses on the geographical part of cadastral data In the INSPIRE context cadastral
        parcels will be mainly used as locators for geo information in general including environmental data. As
        much as possible in the INSPIRE context cadastral parcels should be forming a partition of national territory
        Cadastral parcel should be considered as a single area of Earth surface under homogeneous real property
        rights and unique ownership adapted from UN ECE 2004 and WG CPI 2006 Remark By unique ownership is meant
        that the ownership is held by one or several owners for the whole parcel By homogeneous property rights
        is meant that rights of ownership leases and mortgages affect the whole parcel This does not apply to
        specific rights as servitudes which may only affect part of the parcel

        In the definition given by the INSPIRE directive or equivalent refers to all public agencies and institutions
        other than the main traditional nominal cadastre or land registry that register parts of the Earth s surface
        such as special domains urban cadastres public lands which spatially complement the registrations by the main
        cadastre or land registry.

        Cadastral parcels are considered in the INSPIRE scope if they are available as vector data
        Rights and owners are out of the INSPIRE scope
        Buildings land use addresses are considered in other INSPIRE themes

    """

    label = models.CharField(
        max_length=32,
        blank=True,
        help_text=_(
            "Text commonly used to display the cadastral parcel identification.",
        ),
    )

    national_cadastral_reference = models.CharField(
        max_length=32,
        db_index=True,
        help_text=_(
            "Thematic identifier at national level, generally the full national code of the cadastral "
            "parcel. Must ensure the link to the national cadastral register or equivalent.",
        ),
    )

    area_value = models.IntegerField(
        blank=True,
        null=True,
        help_text=_(
            "Registered area value giving quantification of the area projected on the horizontal plane of "
            "the cadastral parcel.",
        ),
    )

    area_value_uom = models.ForeignKey(
        UnitOfMeasure,
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        limit_choices_to={"measure_type": UnitOfMeasure.MEASURE_TYPE_AREA},
        related_name="%(app_label)s_%(class)s_area_value_uom",
        related_query_name="%(app_label)s_%(class)s_area_value_uoms",
    )

    valid_from = models.DateTimeField(
        blank=True,
        null=True,
        help_text=_(
            "Official date and time the cadastral parcel was/will be legally established.",
        ),
    )

    valid_to = models.DateTimeField(
        blank=True,
        null=True,
        help_text=_(
            "Date and time at which the cadastral parcel legally ceased/will cease to be used.",
        ),
    )

    geometry = models.MultiPolygonField(
        srid=INSPIRE_EU_DEFAULT_SRID,
        blank=True,
        null=True,
        help_text=_(_("Geometry of the cadastral parcel.")),
    )

    reference_point = models.PointField(
        srid=INSPIRE_EU_DEFAULT_SRID,
        blank=True,
        null=True,
        help_text=_(
            "A point within the cadastral parcel. EXAMPLE The centroid of the cadastral parcel geometry.",
        ),
    )

    class Meta:
        abstract = True



import logging

from django.contrib.gis.db import models
from django.utils.translation import ugettext_lazy as _
from slugify import slugify

from ..utils import get_inspire_eu_base_model

log = logging.getLogger(__name__)


###############################################################################
#                             ____                                            #
#                            / ___|___  _ __ ___                              #
#                           | |   / _ \| '__/ _ \                             #
#                           | |__| (_) | | |  __/                             #
#                            \____\___/|_|  \___|                             #
#                                                                             #
###############################################################################

try:
    BaseInspireEUModel = get_inspire_eu_base_model()
except AttributeError:

    class BaseInspireEUModel(models.Model):
        """Base Model

        Definition
            Abstract class that appends common fields
        """

        class Meta:
            abstract = True


class Namespace(BaseInspireEUModel):
    """Namespace

    Definition
        Namespace uniquely identifying the data source of the spatial object.

    Description
        NOTE The namespace value will be owned by the data provider of the spatial object and will be registered
        in the INSPIRE External Object Identifier Namespaces Register.

    """

    code = models.CharField(max_length=32, help_text=_("Namespace"))
    name = models.CharField(max_length=64, blank=True)

    class Meta:
        verbose_name = _("Namespace")
        verbose_name_plural = _("Namespaces")
        ordering = ["code", "name"]

    def __str__(self):
        return "%s %s" % (self.code, self.name)


class Status(BaseInspireEUModel):
    """Status

    Description
        Possible values: Valid, Invalid

    References
        * https://inspire.ec.europa.eu/registry/status/valid
        * https://inspire.ec.europa.eu/registry/status/invalid

    """

    code = models.CharField(max_length=32, db_index=True)
    slug = models.CharField(max_length=32, blank=True, db_index=True)
    label = models.CharField(max_length=128)
    definition = models.TextField(blank=True)
    link = models.URLField()
    is_valid = models.BooleanField(default=True)

    class Meta:
        verbose_name = _("Status")
        verbose_name_plural = _("Status")
        ordering = ["is_valid", "label"]

    def __str__(self):
        return "%s: %s" % (self.code, self.label)

    def save(self, *args, **kwargs):
        self.link = self.link.replace("http://", "https://")
        if not self.code:
            self.code = self.link.split("/")[-1]
        self.slug = slugify(self.code)[:32]
        return super().save(*args, **kwargs)


class Theme(BaseInspireEUModel):
    """INSPIRE theme register

    Definition
        The INSPIRE theme register contains all spatial data themes, as defined in the Annexes of the
        INSPIRE Directive (Directive 2007/2/EC of the European Parliament and of the Council of 14 March 2007
        establishing an Infrastructure for Spatial Information in the European Community (INSPIRE)). The
        descriptions of the themes are based on version 3.0 of the Definition of Annex Themes and Scope (D 2.3)
        by the data specifications drafting team and subsequent updates by the INSPIRE Thematic Working Groups (TWGs).

    Owner
        European Union

    Register manager
        European Commission, Joint Research Centre

    Control body
        Control body for the central INSPIRE registers and INSPIRE register federation

    Submitter
        Nominated submitting organizations for the central INSPIRE registers and INSPIRE register federation

    Contact point
        JRC INSPIRE Registry Team <inspire-registry-dev@jrc.ec.europa.eu>

    Licence
        Europa Legal Notice

    References
        https://inspire.ec.europa.eu/theme
    """

    code = models.CharField(max_length=32, db_index=True)
    slug = models.CharField(max_length=32, blank=True, db_index=True)
    link = models.URLField()
    version = models.SmallIntegerField(blank=True, default=0)
    status = models.ForeignKey(Status, on_delete=models.PROTECT)
    label = models.CharField(max_length=128)
    definition = models.TextField(blank=True)
    description = models.TextField(blank=True)

    class Meta:
        verbose_name = _("Theme")
        verbose_name_plural = _("Themes")
        ordering = ["code", "label"]

    def __str__(self):
        return "%s: %s" % (self.code, self.label)

    def save(self, *args, **kwargs):
        self.link = self.link.replace("http://", "https://")
        if not self.code:
            self.code = self.link.split("/")[-1]
        self.slug = slugify(self.code)[:32]
        return super().save(*args, **kwargs)


class ApplicationSchema(BaseInspireEUModel):
    """INSPIRE Application schema register

    Definition
        The INSPIRE application schema register contains all application schemas of the consolidated
        INSPIRE UML data model.

    Owner
        European Union

    Register manager
        European Commission, Joint Research Centre

    Control body
        Control body for the central INSPIRE registers and INSPIRE register federation

    Submitter
        Nominated submitting organizations for the central INSPIRE registers and INSPIRE register federation

    Contact point
        JRC INSPIRE Registry Team <inspire-registry-dev@jrc.ec.europa.eu>

    Licence
        Europa Legal Notice

    References
        https://inspire.ec.europa.eu/applicationschema
    """

    code = models.CharField(max_length=32, db_index=True)
    slug = models.CharField(max_length=32, blank=True, db_index=True)
    link = models.URLField()
    version = models.SmallIntegerField(blank=True, default=0)
    status = models.ForeignKey(Status, on_delete=models.PROTECT)
    label = models.CharField(max_length=128)
    definition = models.TextField(blank=True)
    description = models.TextField(blank=True)
    themes = models.ManyToManyField(Theme)

    class Meta:
        verbose_name = _("Application Schema")
        verbose_name_plural = _("Application Schemes")
        ordering = ["code", "label"]

    def __str__(self):
        return "%s: %s" % (self.code, self.label)

    def save(self, *args, **kwargs):
        self.link = self.link.replace("http://", "https://")
        if not self.code:
            self.code = self.link.split("/")[-1]
        self.slug = slugify(self.code)[:32]
        return super().save(*args, **kwargs)


class CodeList(BaseInspireEUModel):
    """INSPIRE Code List Register

    Definition
        The INSPIRE code list register contains the code lists and their values, as defined in the INSPIRE
        implementing rules on interoperability of spatial data sets and services (Commission Regulation
        (EU) No 1089/2010).

        NOTE: It does not yet include references to external code lists and the additional code lists and extended
        values proposed in the Data Specification Technical Guidelines.

    Owner
        European Union

    Register manager
        European Commission, Joint Research Centre

    Control body
        Control body for the central INSPIRE registers and INSPIRE register federation

    Submitter
        Nominated submitting organizations for the central INSPIRE registers and INSPIRE register federation

    Contact point
        JRC INSPIRE Registry Team <inspire-registry-dev@jrc.ec.europa.eu>

    Licence
        Europa Legal Notice https://ec.europa.eu/info/legal-notice_en

    References
        https://inspire.ec.europa.eu/codelist

    """

    application_schema = models.ForeignKey(ApplicationSchema, on_delete=models.PROTECT)
    code = models.CharField(max_length=64, db_index=True)
    slug = models.CharField(max_length=64, blank=True, db_index=True)
    link = models.URLField(blank=True)
    status = models.ForeignKey(Status, on_delete=models.PROTECT)
    label = models.CharField(max_length=128)
    definition = models.TextField(blank=True)
    description = models.TextField(blank=True)
    parent = models.ForeignKey("self", blank=True, null=True, on_delete=models.PROTECT)
    themes = models.ManyToManyField(Theme)

    class Meta:
        verbose_name = _("Code list")
        verbose_name_plural = _("Code lists")
        ordering = ["code", "label"]

    def __str__(self):
        return "%s: %s" % (self.code, self.label)

    def save(self, *args, **kwargs):
        self.link = self.link.replace("http://", "https://")
        if not self.code:
            self.code = self.link.split("/")[-1]
        self.slug = slugify(self.code)[:64]
        return super().save(*args, **kwargs)


class CodeListValue(BaseInspireEUModel):
    code_list = models.ForeignKey(CodeList, on_delete=models.PROTECT)
    code = models.CharField(max_length=96, db_index=True)
    slug = models.CharField(max_length=96, blank=True, db_index=True)
    link = models.URLField()
    status = models.ForeignKey(Status, on_delete=models.PROTECT)
    label = models.CharField(max_length=200)
    definition = models.TextField(blank=True)
    description = models.TextField(blank=True)

    class Meta:
        verbose_name = _("Code list value")
        verbose_name_plural = _("Code list values")
        ordering = ["code_list__code", "code", "label"]

    def __str__(self):
        return "%s (%s): %s" % (self.code, self.code_list.code, self.label)

    def save(self, *args, **kwargs):
        if self.link:
            self.link = self.link.replace("http://", "https://")
        if not self.code:
            self.code = self.link.split("/")[-1]
        self.slug = slugify(self.code)[:96]
        return super().save(*args, **kwargs)

    @classmethod
    def search(cls, slug, code_list_slug=None, create=True):
        """Search CodeListValue

        Args:
            slug (str): Slug of CodeList
            code_list_slug (str, optional): Slug of CodeList foreign key. Defaults to None.
            create (bool, optional): If it does not exist, create a new one. Defaults to True.

        Raises:
            CodeList.DoesNotExist: When his foreign key does not exist
            CodeListValue.DoesNotExist: Only when `create` param is false

        Returns:
            CodeListValue: Row founded, or created
        """
        clv = None
        slug_list = slug.split("/")
        if not code_list_slug:
            code_list_slug = slug_list[-2]
        kw = dict(
            {
                "code_list__slug": slugify(code_list_slug),
                "slug": slugify(slug_list[-1]),
            },
        )
        try:
            clv = CodeListValue.objects.get(**kw)
        except CodeListValue.DoesNotExist:
            if create:
                kw_new = dict(
                    {
                        "code": slug_list[-1],
                        "label": slug_list[-1],
                        "definition": slug_list[-1],
                        "description": _("Created ad-hoc"),
                    },
                )
                try:
                    kw_new["code_list"] = CodeList.objects.get(
                        slug=kw["code_list__slug"],
                    )
                except CodeList.DoesNotExist:
                    msg = _(f"There is no CodeList with code '{code_list_slug}'")
                    raise CodeList.DoesNotExist(msg)
                kw_new["status"] = Status.objects.get(slug="valid")
                clv = CodeListValue.objects.create(**kw_new)
                msg = _(f"Created new CodeListValue: '{slug}' at '{code_list_slug}'")
                log.warning(msg)
            else:
                msg = _(f"There is no CodeListValue '{slug}' at '{code_list_slug}'")
                raise CodeListValue.DoesNotExist(msg)
        return clv


class UnitOfMeasure(BaseInspireEUModel):
    """Unit Of Measure

    Definition
        Any of the systems devised to measure some physical quantity such distance or area or a system devised
        to measure such things as the passage of time.

        The classes of UnitOfMeasure are determined by the member "measureType." Subclasses are not needed for
        implementation, but their use makes type constraints on measure valued attributes easier to specify.
        -- conversionToISOstandardUnit is not null only if the conversion is a simple scale

    References
        * `UnitOfMeasure Class <https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=1:3:7:1:1:1:1397>`_
        * `MeasureType Class <https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=1:3:7:1:1:1:1392>`_
    """  # noqa

    MEASURE_TYPE_UNKNOWN = ""
    MEASURE_TYPE_AREA = _("area")
    MEASURE_TYPE_LENGTH = _("length")
    MEASURE_TYPE_ANGLE = _("angle")
    MEASURE_TYPE_TIME = _("time")
    MEASURE_TYPE_VELOCITY = _("velocity")
    MEASURE_TYPE_VOLUME = _("volume")
    MEASURE_TYPE_SCALE = _("scale")
    MEASURE_TYPE_WEIGHT = _("weight")
    MEASURE_TYPE_CHOICES = (
        (MEASURE_TYPE_UNKNOWN, _("Unknown")),
        (MEASURE_TYPE_AREA, _("Area")),
        (MEASURE_TYPE_LENGTH, _("Length")),
        (MEASURE_TYPE_ANGLE, _("Angle")),
        (MEASURE_TYPE_TIME, _("Time")),
        (MEASURE_TYPE_VELOCITY, _("Velocity")),
        (MEASURE_TYPE_VOLUME, _("Volume")),
        (MEASURE_TYPE_SCALE, _("Scale")),
        (MEASURE_TYPE_WEIGHT, _("Weight")),
    )

    name = models.CharField(
        max_length=32,
        blank=True,
        help_text=_(
            "The name(s) of a particular unit of measure. Examples would include the following: "
            "1) for uomArea - square feet <br />"
            "2) for uomTime - seconds <br />"
            "3) for uomArea - miles<br />"
            "4) uomAngle - degrees.",
        ),
    )
    slug = models.CharField(max_length=32, blank=True, db_index=True)
    symbol = models.CharField(
        max_length=8,
        help_text=_(
            """The symbol used for this unit of measure, such at "ft" for feet, or "m" for meter.""",
        ),
    )
    measure_type = models.CharField(
        max_length=8,
        choices=MEASURE_TYPE_CHOICES,
        help_text=_("Measure Type"),
    )
    name_standard_unit = models.CharField(
        max_length=8,
        blank=True,
        help_text=_(
            "Name of the standard units to which this unit of measure can be directly converted. "
            "If this variable is NULL, then the standard unit for this measure type given by the local "
            "copy of the StandardsUnits code list.",
        ),
    )
    scale_to_standard_unit = models.FloatField(
        blank=True,
        null=True,
        help_text=_(
            "If the implementation system used for this object does not support NULL, the  scale set to 0 "
            "is equivalent to NULL for both scale and offset.<br />"
            "If X is the current unit, and S is the standard one the of two variables scale(ToStandardUnit) "
            "and offset(ToStandardUnit) can be used to make the conversion from X to S by:<br />"
            "S = offset + scale*X <br />"
            "and, conversely, <br />"
            "X = (S-offset)/scale",
        ),
    )
    offset_to_standard_unit = models.FloatField(
        blank=True,
        null=True,
        help_text=_(
            "See scaleToStandardUnit for a description. Again, this variable is NULL is no linear conversion "
            "is possible. If the two units are only a scale in difference, then this number is zero (0). "
            "If the implementation system used for this object does not support NULL, the then scale set "
            "to 0 is equivalent to NULL for both scale and offset.",
        ),
    )
    formula = models.CharField(
        blank=True,
        max_length=32,
        help_text=_(
            "An algebraic formula (probably in some programming language) converting this unit of measure "
            "(represented in the formula by its uomSymbol) to the ISO standard (represented by its symbol. "
            "This member attribute is not required, but it is a valuable piece of documentation. ",
        ),
    )

    # objects = models.Manager()
    # uom_length = models.Manager()
    # uom_area = models.Manager()
    # uom_length = models.Manager()
    # uom_angle = models.Manager()
    # uom_time = models.Manager()
    # uom_velocity = models.Manager()
    # uom_volume = models.Manager()
    # uom_scale = models.Manager()
    # uom_weight = models.Manager()

    class Meta:
        verbose_name = _("Unit Of Measure")
        verbose_name_plural = _("Units Of Measure")
        ordering = ["symbol", "name"]

    def __str__(self):
        return "%s %s" % (self.symbol, self.name)

    def save(self, *args, **kwargs):
        self.slug = slugify(self.name)[:32]
        return super().save(*args, **kwargs)



"""================
Default Settings
================

INSPIRE_EU_DEFAULT_SRID
-----------------------

.. code-block:: python

    INSPIRE_EU_DEFAULT_SRID = settings.INSPIRE_EU_DEFAULT_SRID

INSPIRE_EU_THEMES
-----------------

.. code-block:: python

    INSPIRE_EU_THEMES = {
        "cadastral_parcels": True,
        "buildings": True,
    }

"""
import logging

from django.conf import settings

log = logging.getLogger(__name__)

try:
    INSPIRE_EU_DEFAULT_SRID = settings.INSPIRE_EU_DEFAULT_SRID
except AttributeError:
    INSPIRE_EU_DEFAULT_SRID = 4326

try:
    INSPIRE_EU_THEMES = settings.INSPIRE_EU_THEMES
except AttributeError:
    INSPIRE_EU_THEMES = {
        "cadastral_parcels": True,
        "buildings": True,
    }


from .core import (  # noqa
    ApplicationSchema,
    BaseInspireEUModel,
    CodeList,
    CodeListValue,
    Namespace,
    Status,
    Theme,
    UnitOfMeasure,
)



# -*- coding: utf-8 -*-
"""**************
INSPIRE themes
**************

The INSPIRE theme register contains all spatial data themes, as defined in the Annexes of the INSPIRE Directive
(Directive 2007/2/EC of the European Parliament and of the Council of 14 March 2007 establishing an Infrastructure
for Spatial Information in the European Community (INSPIRE)). The descriptions of the themes are based on version
3.0 of the Definition of Annex Themes and Scope (D 2.3) by the data specifications drafting team and subsequent
updates by the INSPIRE Thematic Working Groups (TWGs).

.. automodule:: inspire_eu.models

=============
Common Models
=============

.. automodule:: inspire_eu.models.abstract
   :members: BaseModel,Identifier,DataLifeCycleInfo,AbstractGeographicalName
   :member-order: bysource
   :inherited-members:
   :exclude-members: __init__,clean,clean_fields,full_clean,get_deferred_fields,refresh_from_db,save,save_base,serializable_value,validate_unique,DoesNotExist, MultipleObjectsReturned

=============
Core Models
=============

.. automodule:: inspire_eu.models.core
   :members: Status,Namespace,Theme,ApplicationSchema,CodeList,CodeListValue,UnitOfMeasure
   :member-order: bysource
   :noindex:
   :inherited-members:
   :exclude-members: __init__,clean,clean_fields,full_clean,get_deferred_fields,refresh_from_db,save,save_base,serializable_value,validate_unique,DoesNotExist, MultipleObjectsReturned,BaseModel

"""  # noqa

__version__ = "0.2.2"



import logging

from django.contrib.gis.db import models
from django.utils.translation import ugettext_lazy as _

from .core import CodeListValue, Namespace

log = logging.getLogger(__name__)


###############################################################################
#                    _    _         _                  _                      #
#                   / \  | |__  ___| |_ _ __ __ _  ___| |_                    #
#                  / _ \ | '_ \/ __| __| '__/ _` |/ __| __|                   #
#                 / ___ \| |_) \__ \ |_| | | (_| | (__| |_                    #
#                /_/   \_\_.__/|___/\__|_|  \__,_|\___|\__|                   #
#                                                                             #
###############################################################################


class Identifier(models.Model):
    """Identifier

    Definition
        External unique object identifier published by the responsible body, which may be used by external
        applications to reference the spatial object.

    Description

        NOTE 1 External object identifiers are distinct from thematic object identifiers.

        NOTE 2 The voidable version identifier attribute is not part of the unique identifier of a spatial object and
        may be used to distinguish two versions of the same spatial object.

    """

    namespace = models.ForeignKey(
        Namespace,
        on_delete=models.PROTECT,
        related_name="%(app_label)s_%(class)s_namespace",
        related_query_name="%(app_label)s_%(class)s_namespaces",
    )
    local_id = models.CharField(
        max_length=32,
        help_text=_(
            "A local identifier, assigned by the data provider. The local identifier is unique within the "
            "namespace, that is no other spatial object carries the same unique identifier.<br />"
            "NOTE It is the responsibility of the data provider to guarantee uniqueness of the local "
            "identifier within the namespace.",
        ),
    )
    version_id = models.CharField(
        max_length=25,
        blank=True,
        help_text=_(
            "The identifier of the particular version of the spatial object, with a maximum length of 25 "
            "characters. If the specification of a spatial object type with an external object identifier "
            "includes life-cycle information, the version identifier is used to distinguish between the "
            "different versions of a spatial object. Within the set of all versions of a spatial object, "
            "the version identifier is unique.<br />"
            "NOTE The maximum length has been selected to allow for time stamps based on ISO 8601, "
            """for example, "2007-02-12T12:12:12+05:30" as the version identifier.<br />"""
            "NOTE 2 The property is void, if the spatial data set does not distinguish between different "
            "versions of the spatial object. It is missing, if the spatial object type does not support "
            "any life-cycle information.",
        ),
    )

    class Meta:
        abstract = True
        unique_together = ["namespace", "local_id", "version_id"]


class DataLifeCycleInfo(models.Model):
    """DataLifeCycleInfo

    Definition
        Begin and End datetime fields
    """

    begin_lifespan_version = models.DateTimeField(
        help_text=_(
            "Registered area value giving quantification of the area projected on the horizontal plane of "
            "the cadastral parcel.",
        ),
    )

    end_lifespan_version = models.DateTimeField(
        blank=True,
        null=True,
        help_text=_(
            "Date and time at which this version of the spatial object was superseded or retired in the "
            "spatial data set.",
        ),
    )

    class Meta:
        abstract = True


class AbstractGeographicalName(models.Model):
    """Geographical Name

    Definition
        Proper noun applied to a real world entity.

    References
        `GeographicalName Class <https://inspire.ec.europa.eu/data-model/approved/r4618-ir/html/index.htm?goto=2:1:6:2:7246>`_
    """  # noqa

    language = models.CharField(
        max_length=64,
        blank=True,
        help_text=_(
            "Language of the name, given as a three letters code, in accordance with either "
            "ISO 639-3 or ISO 639-5.",
        ),
    )
    nativeness = models.ForeignKey(
        CodeListValue,
        limit_choices_to={"code_list__code": "NativenessValue"},
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        help_text=_(
            "Information enabling to acknowledge if the name is the one that is/was used in the area where "
            "the spatial object is situated at the instant when the name is/was in use.",
        ),
        related_name="%(app_label)s_%(class)s_nativeness",
        related_query_name="%(app_label)s_%(class)s_nativenesss",
    )
    name_status = models.ForeignKey(
        CodeListValue,
        limit_choices_to={"code_list__code": "NameStatusValue"},
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        help_text=_(
            "Qualitative information enabling to discern which credit should be given to the name "
            "with respect to its standardization and/or its topicality.",
        ),
        related_name="%(app_label)s_%(class)s_name_status",
        related_query_name="%(app_label)s_%(class)s_name_statuss",
    )
    source_of_name = models.CharField(
        max_length=64,
        blank=True,
        help_text=_(
            "Original data source from which the geographical name is taken from and integrated in the data set "
            "providing/publishing it. For some named spatial objects it might refer again to the publishing data "
            "set if no other information is available.",
        ),
    )
    pronunciation = models.CharField(
        max_length=64,
        blank=True,
        help_text=_(
            "Proper, correct or standard (standard within the linguistic community concerned) "
            "pronunciation of the geographical name.",
        ),
    )
    spelling = models.CharField(
        max_length=64,
        blank=True,
        help_text=_("A proper way of writing the geographical name."),
    )
    grammatical_gender = models.ForeignKey(
        CodeListValue,
        limit_choices_to={"code_list__code": "GrammaticalGenderValue"},
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        help_text=_("Class of nouns reflected in the behavior of associated words."),
        related_name="%(app_label)s_%(class)s_grammatical_gender",
        related_query_name="%(app_label)s_%(class)s_grammatical_genders",
    )
    grammatical_number = models.ForeignKey(
        CodeListValue,
        limit_choices_to={"code_list__code": "GrammaticalNumberValue"},
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        help_text=_("Grammatical category of nouns that expresses count distinctions."),
        related_name="%(app_label)s_%(class)s_grammatical_number",
        related_query_name="%(app_label)s_%(class)s_grammatical_numbers",
    )

    class Meta:
        abstract = True



# Standard Library
import logging

import feedparser
import requests
from django.core.management.base import BaseCommand
from django.utils.translation import ugettext as _
from slugify import slugify

from ...models import ApplicationSchema, CodeList, CodeListValue, Status, Theme

log = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Load initial data"
    base_url = "https://inspire.ec.europa.eu"
    debug_console = None

    def add_arguments(self, parser):
        parser.add_argument(
            "-l",
            "--language",
            type=str,
            help=_("Language (default: en)"),
        )

    def populate_status(self, language):
        if self.debug_console:
            print("*********************")
            print("* Populating Status *")
            print("*********************")
        status_list = [
            {
                "link": "https://inspire.ec.europa.eu/registry/status/valid",
                "label": _("Valid"),
                "definition": _(
                    "The item has been accepted, is recommended for use, and has not been "
                    "superseded or retired.",
                ),
                "is_valid": True,
            },
            {
                "link": "https://inspire.ec.europa.eu/registry/status/invalid",
                "label": _("Invalid"),
                "definition": _(
                    "A decision has been made that a previously valid register item contains a substantial "
                    "error and is invalid, and will normally have been replaced by a corrected item.",
                ),
                "is_valid": False,
            },
            {
                "link": "https://inspire.ec.europa.eu/registry/status/retired",
                "label": _("Retired"),
                "definition": _(
                    "A decision has been made that the item is no longer recommended for use. "
                    "It has not been superseded by another item.",
                ),
                "is_valid": False,
            },
        ]
        for status_dict in status_list:
            update = False
            code = status_dict["link"].split("/")[-1]
            try:
                status = Status.objects.get(slug=slugify(code))
            except Status.DoesNotExist:
                status = Status(code=code)
                update = True

            for key, value in status_dict.items():
                # value = status_dict[key]
                if getattr(status, key) != value:
                    setattr(status, key, value)
                    update = True
            if update:
                status.save()
                if self.debug_console:
                    print(f"Updated status: '{status}'")

        print("End Populating Status")
        print("")

    def populate_themes(self, language):
        if self.debug_console:
            print("********************")
            print("* Populating Theme *")
            print("********************")
        response = requests.get(url=f"{self.base_url}/theme/theme.{language}.json")
        data = response.json()
        themes = data["register"]["containeditems"]
        for theme_dict in themes:
            theme_json = theme_dict["theme"]
            code = theme_json["id"].split("/")[-1]
            row = dict({"code": code, "link": theme_json["id"]})
            try:
                status = Status.objects.get(
                    slug=slugify(theme_json["status"]["id"].split("/")[-1]),
                )
            except Status.DoesNotExist:
                status_link = theme_json["status"]["id"]
                try:
                    status_label = theme_json["status"]["label"]["text"]
                except KeyError:
                    status_label = status_link
                status = Status(link=status_link, label=status_label)
                status.save()
            row["status"] = status
            try:
                row["version"] = int(theme_json["version"])
            except (TypeError, ValueError):
                msg = _(f"""{row["code"]} Version inválida: {theme_json["version"]}""")
                log.warning(msg)
                row["version"] = 0

            for f in ["label", "definition", "description"]:
                try:
                    row[f] = theme_json[f]["text"].strip()
                except KeyError:
                    row[f] = ""

            update = False
            try:
                theme = Theme.objects.get(code=code)
            except Theme.DoesNotExist:
                theme = Theme(code=code, status=status)
                update = True

            for f in ["link", "version", "label", "definition", "description"]:
                if getattr(theme, f) != row[f]:
                    setattr(theme, f, row[f])
                    update = True

            if update:
                theme.save()
                if self.debug_console:
                    print(f"Updated Theme '{theme}'")

        print("End Populating Theme")
        print("")

    def populate_schemas(self, language):
        if self.debug_console:
            print("*********************************")
            print("* Populating Application Schema *")
            print("*********************************")
        response = requests.get(
            url=f"{self.base_url}/applicationschema/applicationschema.{language}.json",
        )
        data = response.json()
        schemas = data["register"]["containeditems"]
        for schema_dict in schemas:
            schema_json = schema_dict["applicationschema"]
            code = schema_json["id"].split("/")[-1]
            row = dict({"code": code, "link": schema_json["id"]})
            try:
                status = Status.objects.get(
                    slug=slugify(schema_json["status"]["id"].split("/")[-1]),
                )
            except Status.DoesNotExist:
                status_link = schema_json["status"]["id"]
                try:
                    status_label = schema_json["status"]["label"]["text"].strip()
                except KeyError:
                    status_label = status_link
                status = Status(link=status_link, label=status_label)
                status.save()
            row["status"] = status
            try:
                row["version"] = int(schema_json["version"])
            except (TypeError, ValueError):
                msg = _(f"""{row["code"]} Version inválida: {schema_json["version"]}""")
                log.warning(msg)
                row["version"] = 0

            for f in ["label", "definition", "description"]:
                try:
                    row[f] = schema_json[f]["text"].strip()
                except KeyError:
                    row[f] = ""

            update = False
            try:
                schema = ApplicationSchema.objects.get(slug=slugify(code))
            except ApplicationSchema.DoesNotExist:
                schema = ApplicationSchema(code=code, status=status)
                update = True

            for f in ["link", "version", "label", "definition", "description"]:
                if getattr(schema, f) != row[f]:
                    setattr(schema, f, row[f])
                    update = True

            if update:
                schema.save()
                if self.debug_console:
                    print(f"Updated Application Schema: '{schema}'")

            try:
                for theme_json in schema_json["themes"]:
                    theme_dict = theme_json["theme"]
                    theme_code = theme_dict["id"].split("/")[-1]
                    try:
                        theme = Theme.objects.get(slug=slugify(theme_code))
                    except Theme.DoesNotExist:
                        theme_link = theme_dict["id"]
                        try:
                            theme_label = theme_dict["label"]["text"]
                        except KeyError:
                            theme_label = theme_link
                        theme = Theme(link=theme_link, label=theme_label, status=status)
                        theme.save()

                    if theme not in schema.themes.all():
                        schema.themes.add(theme)
            except KeyError:
                theme = None
        print("End Application Schema")
        print("")

    def populate_code_list(self, language):
        if self.debug_console:
            print("***********************")
            print("* Populating CodeList *")
            print("***********************")
        url = f"{self.base_url}/codelist/codelist.{language}.atom"

        if self.debug_console:
            print(f">>> Fetching {url}")
        response = requests.get(url=url)
        fp = feedparser.parse(response.text)
        status = Status.objects.get(slug="valid")
        for entry in fp.entries:
            code = entry["id"].split("/")[-1]
            row = dict({"code": code})
            row["link"] = entry["link"].strip()
            row["label"] = entry["title"].strip()
            row["definition"] = entry["summary"].strip()
            try:
                row["description"] = entry["content"][0]["value"]
            except KeyError:
                row["description"] = ""

            themes = []
            row["application_schema"] = None
            for entry_link in entry["links"]:
                if entry_link["rel"] == "self":
                    continue
                elif entry_link["rel"] == "up":
                    if entry_link["href"] == "http://inspire.ec.europa.eu/codelist":
                        continue
                    else:
                        raise Exception("Check!")
                elif entry_link["rel"] == "related":
                    if "theme" in entry_link["href"]:
                        theme_slug = slugify(entry_link["href"].split("/")[-1])
                        try:
                            themes.append(Theme.objects.get(slug=theme_slug))
                        except Theme.DoesNotExist:
                            pass
                    elif "applicationschema" in entry_link["href"]:
                        schema_slug = slugify(entry_link["href"].split("/")[-1])
                        try:
                            row["application_schema"] = ApplicationSchema.objects.get(
                                slug=schema_slug,
                            )
                        except ApplicationSchema.DoesNotExist:
                            pass

            update = False
            try:
                code_list = CodeList.objects.get(slug=slugify(code))
            except CodeList.DoesNotExist:
                code_list = CodeList(code=code, status=status)
                update = True

            for key, value in row.items():
                try:
                    if getattr(code_list, key) != value:
                        setattr(code_list, key, value)
                        update = True
                except AttributeError:
                    setattr(code_list, key, value)
                    update = True

            if update:
                code_list.save()
                if self.debug_console:
                    print(f"Updated CodeList: '{code_list}'")
                for theme in themes:
                    if theme not in code_list.themes.all():
                        code_list.themes.add(theme)

        print("End CodeList")
        print("")

    def populate_code_values(self, language):
        if self.debug_console:
            print("****************************")
            print("* Populating CodeListValue *")
            print("****************************")
        errors = dict({"key": [], "data": [], "multiple_parents": [], "no_data": []})
        qs = CodeList.objects.all()
        error_keys = [
            "SpecificAppurtenanceTypeValue",
            "ClassificationItemTypeValue",
            "PartyRoleValue",
            "PhenomenonTypeValue",
            "InputOutputValue",
            "ActivityCodeValue",
            "EconomicActivityValue",
            "ReferenceHabitatTypeCodeValue",
            "ReferenceSpeciesCodeValue",
            "RegionClassificationValue",
            "CountingUnitValue",
            "DesignationValue",
        ]
        error_keys = ["ClassificationItemTypeValue"]  # noqa
        # qs = qs.filter(code__in=error_keys)
        for code_list in qs:
            url = code_list.link
            url += f"/{code_list.code}.{language}.json"
            if self.debug_console:
                print(f">>> Fetching {url}")
            response = requests.get(url=url)
            if response.status_code == 404:
                continue
            data = response.json()
            try:
                item_dict_list = data["codelist"]["containeditems"]
            except KeyError:
                # No data # ToDo ¿?¿?
                if url not in errors["no_data"]:
                    errors["no_data"].append(url)
                continue
            for item_dict in item_dict_list:
                try:
                    item = item_dict["value"]
                except KeyError:
                    # item_code_list = item_dict["codelist"] # ToDo ¿?¿?
                    if url not in errors["key"]:
                        errors["key"].append(url)
                    continue

                row = dict()
                if "parents" in item:
                    parents = item["parents"]
                    if len(parents) > 1:
                        # https://inspire.ec.europa.eu/codelist/CommodityCodeValue/limestone
                        if url not in errors["multiple_parents"]:
                            errors["multiple_parents"].append(item["id"])

                    parent = parents[0]["parent"]
                    row["parent"] = CodeListValue.search(parent["id"])

                code = item["id"].split("/")[-1]
                status_link = item["status"]["id"]
                try:
                    status = Status.objects.get(
                        slug=slugify(status_link.split("/")[-1]),
                    )
                except Status.DoesNotExist:
                    try:
                        status_label = item["status"]["label"]["text"].strip()
                    except KeyError:
                        status_label = status_link
                    status = Status(link=status_link, label=status_label)
                    status.save()
                row["status"] = status
                row["link"] = item["id"]

                update = False
                try:
                    code_list_value = CodeListValue.objects.get(
                        code_list=code_list,
                        slug=slugify(code),
                    )
                except CodeListValue.DoesNotExist:
                    code_list_value = CodeListValue(code_list=code_list, code=code)
                    update = True

                for f in ["label", "definition", "description"]:
                    try:
                        row[f] = item[f]["text"].strip()
                    except KeyError:
                        row[f] = ""

                for key, value in row.items():
                    try:
                        if getattr(code_list_value, key) != value:
                            setattr(code_list_value, key, value)
                            update = True
                    except AttributeError:
                        setattr(code_list_value, key, value)
                        update = True

                if update:
                    try:
                        code_list_value.save()
                        if self.debug_console:
                            print(f"Updated CodeListValue: '{code_list_value}'")
                    except Exception as e:
                        errors["data"].append(f"{code}: {e}")
        # print("\n".join(errors["key"]))
        print("End CodeListValue")
        print("")

    def check_language(self, language):
        LANGUAGES_AVAILABLE = dict(
            {
                "bg": "български",
                "cs": "čeština",
                "da": "dansk",
                "de": "deutsch",
                "et": "eesti keel",
                "el": "ελληνικά",
                "en": "english",
                "es": "español",
                "fr": "français",
                "hr": "hrvatski",
                "it": "italiano",
                "lv": "latviešu valoda",
                "lt": "lietuvių kalba",
                "hu": "magyar",
                "mt": "malti",
                "nl": "nederlands",
                "pl": "polski",
                "pt": "português",
                "ro": "română",
                "sk": "slovenčina",
                "sl": "slovenščina",
                "fi": "suomi",
                "sv": "svenska",
            },
        )
        try:
            _lang = LANGUAGES_AVAILABLE[language]  # noqa
        except KeyError:
            print()
            print(f"ERROR: Language '{language}' is not availbale")
            print()
            print("These are current available languages:")
            print()
            for k, v in LANGUAGES_AVAILABLE.items():
                print(f"    {k}: {v}")
            print()
            print()
            language = None
        return language

    def handle(self, *args, **kwargs):
        if kwargs.get("verbosity") > 0:
            self.debug_console = True
        else:
            self.debug_console = False
        language = kwargs.get("language")
        if language:
            language = self.check_language(language)
            if not language:
                return
        else:
            language = "en"

        self.populate_status(language)
        self.populate_themes(language)
        self.populate_schemas(language)
        self.populate_code_list(language)
        self.populate_code_values(language)



import logging

from django.contrib.gis import admin

from . import (
    Building,
    BuildingCurrentUse,
    BuildingDocument,
    BuildingElevation,
    BuildingExternalReference,
    BuildingGeographicalName,
    BuildingHeightAboveGround,
    BuildingNature,
    OtherConstruction,
)

log = logging.getLogger(__name__)


@admin.register(Building)
class BuildingAdmin(admin.OSMGeoAdmin):
    pass


@admin.register(BuildingDocument)
class BuildingDocumentAdmin(admin.ModelAdmin):
    pass


@admin.register(BuildingElevation)
class BuildingElevationAdmin(admin.ModelAdmin):
    pass


@admin.register(BuildingExternalReference)
class BuildingExternalReferenceAdmin(admin.ModelAdmin):
    pass


@admin.register(BuildingHeightAboveGround)
class BuildingHeightAboveGroundAdmin(admin.ModelAdmin):
    pass


@admin.register(BuildingNature)
class BuildingNatureAdmin(admin.ModelAdmin):
    pass


@admin.register(BuildingCurrentUse)
class BuildingCurrentUseAdmin(admin.ModelAdmin):
    pass


@admin.register(BuildingGeographicalName)
class BuildingGeographicalNameAdmin(admin.ModelAdmin):
    pass


@admin.register(OtherConstruction)
class OtherConstructionAdmin(admin.ModelAdmin):
    pass



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


def get_inspire_eu_base_model(inspire_base_model=None):
    """
    Return the Base Model
    """

    if inspire_base_model is None:
        inspire_base_model = settings.INSPIRE_EU_BASE_MODEL

    try:
        module_name, class_name = inspire_base_model.rsplit(".", 1)
        _models = __import__(module_name, fromlist="*")
        try:
            return getattr(_models, class_name)
        except AttributeError as e1:
            raise ImproperlyConfigured(f"{inspire_base_model}: {e1}")
    except AttributeError as e2:
        raise ImproperlyConfigured(f"{inspire_base_model}: {e2}")
    except ModuleNotFoundError as e3:
        raise ImproperlyConfigured(f"{inspire_base_model}: {e3}")
